from math import sqrt
import scipy.sparse.linalg as sla
import matplotlib.pyplot
import kwant

sys = kwant.Builder()
lat = kwant.lattice.general([(1,0,0), (0,1,0), (0,0,1)])

t = 1.0
R = 10

sys[(lat(x,y,z) for x in xrange(-R-1, R+1)
     for y in xrange(-R-1, R+1) for z in xrange(R+1)
     if sqrt(x**2 + y**2 + z**2) < R + 0.01)] = 4 * t
sys[(lat(x,y,z) for x in xrange(-2*R, 2*R + 1)
     for y in xrange(-R, R+1) for z in xrange(-R, 0))] = 4 * t
sys[lat.neighbors()] = -t
sys = sys.finalized()
kwant.plot(sys)

ham = sys.hamiltonian_submatrix(sparse=True)
wf = abs(sla.eigs(ham, sigma=0.2, k=6)[1][:,0])**2
size = 3 * wf / wf.max()
kwant.plot(sys, site_size=size, site_color=(0,0,1,0.05), site_lw=0)



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

import numpy                    # Needed by C. Gohlke's Windows package.

__all__ = ['system', 'version', 'builder', 'lattice', 'solvers',
           'digest', 'rmt']
for module in __all__:
    exec 'from . import {0}'.format(module)

from .version import version as __version__

# Make selected functionality available directly in the root namespace.
available = [('builder', ['Builder', 'HoppingKind']),
             ('lattice', ['TranslationalSymmetry']),
             ('solvers.default',
              ['smatrix', 'greens_function', 'ldos', 'wave_function'])]
for module, names in available:
    exec 'from .{0} import {1}'.format(module, ', '.join(names))
    __all__.extend(names)

# Importing plotter might not work, but this does not have to be a problem --
# only no plotting will be available.
try:
    from . import plotter
    from .plotter import plot
except:
    pass
else:
    __all__.extend(['plotter', 'plot'])


def test(verbose=1):
    from nose.core import run
    import os.path

    return run(argv=[__file__, os.path.dirname(os.path.abspath(__file__)),
                     "-s", "--verbosity="+str(verbose)])

test.__test__ = False



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

import kwant
from kwant.graph import slicer

def assert_sanity(graph, slices):
    # Slices must comprise all of the graph.
    slclist = [slices[j][i] for j in xrange(len(slices))
               for i in xrange(len(slices[j]))]
    slclist.sort()
    assert slclist == [i for i in xrange(graph.num_nodes)]

    # Nodes may only have neighbors in neighboring slices.
    for j in xrange(len(slices)):
        for node in slices[j]:
            for neigh in graph.out_neighbors(node):
                if j > 0 and j < len(slices) - 1:
                    assert (neigh in slices[j] or
                            neigh in slices[j+1] or
                            neigh in slices[j-1])
                elif j == 0:
                    assert (neigh in slices[j] or
                            neigh in slices[j+1])
                else:
                    assert (neigh in slices[j] or
                            neigh in slices[j-1])


def test_rectangle():
    w = 5

    for l in [1, 2, 5, 10]:
        sys = kwant.Builder()
        lead = kwant.Builder(kwant.TranslationalSymmetry((-1, 0)))
        lat = kwant.lattice.square()
        lead[(lat(0, i) for i in xrange(w))] = 0
        sys[(lat(j, i) for j in xrange(l) for i in xrange(w))] = 0
        for s in [lead, sys]:
            for kind in [kwant.builder.HoppingKind((1, 0), lat),
                         kwant.builder.HoppingKind((0, 1), lat)]:
                s[kind] = -1
        sys.attach_lead(lead)
        sys.attach_lead(lead.reversed())
        fsys = sys.finalized()

        slices = slicer.slice(fsys.graph,
                              fsys.lead_interfaces[0],
                              fsys.lead_interfaces[1])

        # In the rectangle case, the slicing is very constricted and
        # we know that all slices must have the same shape.
        assert len(slices) == l

        for j in xrange(l):
            assert len(slices[j]) == w

        assert_sanity(fsys.graph, slices)



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

from __future__ import division
import numpy as np
from itertools import product, izip
from numpy.testing import assert_almost_equal
from kwant.physics import leads
import kwant

modes_se = leads.selfenergy

def h_cell_s_func(t, w, e):
    h = (4 * t - e) * np.identity(w)
    h.flat[1 :: w + 1] = -t
    h.flat[w :: w + 1] = -t
    return h


def test_analytic_numeric():
    w = 5                       # width
    t = 0.78                    # hopping element
    e = 1.3                     # Fermi energy

    assert_almost_equal(leads.square_selfenergy(w, t, e),
                        modes_se(h_cell_s_func(t, w, e), -t * np.identity(w)))


def test_regular_fully_degenerate():
    """Selfenergy with an invertible hopping matrix, and degenerate bands."""

    w = 6                       # width
    t = 0.5                     # hopping element
    e = 1.3                     # Fermi energy

    h_hop_s = -t * np.identity(w)
    h_cell_s = h_cell_s_func(t, w, e)

    h_hop = np.zeros((2*w, 2*w))
    h_hop[:w, :w] = h_hop_s
    h_hop[w:, w:] = h_hop_s

    h_cell = np.zeros((2*w, 2*w))
    h_cell[:w, :w] = h_cell_s
    h_cell[w:, w:] = h_cell_s

    g = np.zeros((2*w, 2*w), dtype=complex)
    g[:w, :w] = leads.square_selfenergy(w, t, e)
    g[w:, w:] = leads.square_selfenergy(w, t, e)

    assert_almost_equal(g, modes_se(h_cell, h_hop))


def test_regular_degenerate_with_crossing():
    """This is a testcase with invertible hopping matrices,
    and degenerate k-values with a crossing such that one
    mode has a positive velocity, and one a negative velocity

    For this case the fall-back technique must be used.
    """

    w = 4                       # width
    t = 0.5                     # hopping element
    e = 1.8                     # Fermi energy

    global h_hop
    h_hop_s = -t * np.identity(w)
    h_cell_s = h_cell_s_func(t, w, e)

    hop = np.zeros((2*w, 2*w))
    hop[:w, :w] = h_hop_s
    hop[w:, w:] = -h_hop_s

    h_cell = np.zeros((2*w, 2*w))
    h_cell[:w, :w] = h_cell_s
    h_cell[w:, w:] = -h_cell_s

    g = np.zeros((2*w, 2*w), dtype=complex)
    g[:w, :w] = leads.square_selfenergy(w, t, e)
    g[w:, w:] = -np.conj(leads.square_selfenergy(w, t, e))

    assert_almost_equal(g, modes_se(h_cell, hop))


def test_singular():
    """This testcase features a rectangular (and hence singular)
     hopping matrix without degeneracies.

    This case can be treated with the Schur technique."""

    w = 5                       # width
    t = .5                     # hopping element
    e = 0.4                     # Fermi energy

    h_hop_s = -t * np.identity(w)
    h_cell_s = h_cell_s_func(t, w, e)

    h_hop = np.zeros((2*w, w))
    h_hop[w:, :w] = h_hop_s

    h_cell = np.zeros((2*w, 2*w))
    h_cell[:w, :w] = h_cell_s
    h_cell[:w, w:] = h_hop_s
    h_cell[w:, :w] = h_hop_s
    h_cell[w:, w:] = h_cell_s
    g = leads.square_selfenergy(w, t, e)

    assert_almost_equal(g, modes_se(h_cell, h_hop))


def test_singular_but_square():
    """This testcase features a singular, square hopping matrices
    without degeneracies.

    This case can be treated with the Schur technique."""

    w = 5                       # width
    t = 0.9                     # hopping element
    e = 2.38                     # Fermi energy

    h_hop_s = -t * np.identity(w)
    h_cell_s = h_cell_s_func(t, w, e)

    h_hop = np.zeros((2*w, 2*w))
    h_hop[w:, :w] = h_hop_s

    h_cell = np.zeros((2*w, 2*w))
    h_cell[:w, :w] = h_cell_s
    h_cell[:w, w:] = h_hop_s
    h_cell[w:, :w] = h_hop_s
    h_cell[w:, w:] = h_cell_s

    g = np.zeros((2*w, 2*w), dtype=complex)
    g[:w, :w] = leads.square_selfenergy(w, t, e)
    assert_almost_equal(g, modes_se(h_cell, h_hop))


def test_singular_fully_degenerate():
    """This testcase features a rectangular (and hence singular)
     hopping matrix with complete degeneracy.

    This case can still be treated with the Schur technique."""

    w = 5                       # width
    t = 1.5                     # hopping element
    e = 3.3                     # Fermi energy

    h_hop_s = -t * np.identity(w)
    h_cell_s = h_cell_s_func(t, w, e)

    h_hop = np.zeros((4*w, 2*w))
    h_hop[2*w:3*w, :w] = h_hop_s
    h_hop[3*w:4*w, w:2*w] = h_hop_s

    h_cell = np.zeros((4*w, 4*w))
    h_cell[:w, :w] = h_cell_s
    h_cell[:w, 2*w:3*w] = h_hop_s
    h_cell[w:2*w, w:2*w] = h_cell_s
    h_cell[w:2*w, 3*w:4*w] = h_hop_s
    h_cell[2*w:3*w, :w] = h_hop_s
    h_cell[2*w:3*w, 2*w:3*w] = h_cell_s
    h_cell[3*w:4*w, w:2*w] = h_hop_s
    h_cell[3*w:4*w, 3*w:4*w] = h_cell_s

    g = np.zeros((2*w, 2*w), dtype=complex)
    g[:w, :w] = leads.square_selfenergy(w, t, e)
    g[w:, w:] = leads.square_selfenergy(w, t, e)

    assert_almost_equal(g, modes_se(h_cell, h_hop))


def test_singular_degenerate_with_crossing():
    """This testcase features a rectangular (and hence singular)
     hopping matrix with degeneracy k-values including a crossing
     with velocities of opposite sign.

    This case must be treated with the fall-back technique."""

    w = 5                       # width
    t = 20.5                     # hopping element
    e = 3.3                     # Fermi energy

    h_hop_s = -t * np.identity(w)
    h_cell_s = h_cell_s_func(t, w, e)

    h_hop = np.zeros((4*w, 2*w))
    h_hop[2*w:3*w, :w] = h_hop_s
    h_hop[3*w:4*w, w:2*w] = -h_hop_s

    h_cell = np.zeros((4*w, 4*w))
    h_cell[:w, :w] = h_cell_s
    h_cell[:w, 2*w:3*w] = h_hop_s
    h_cell[w:2*w, w:2*w] = -h_cell_s
    h_cell[w:2*w, 3*w:4*w] = -h_hop_s
    h_cell[2*w:3*w, :w] = h_hop_s
    h_cell[2*w:3*w, 2*w:3*w] = h_cell_s
    h_cell[3*w:4*w, w:2*w] = -h_hop_s
    h_cell[3*w:4*w, 3*w:4*w] = -h_cell_s

    g = np.zeros((2*w, 2*w), dtype=complex)
    g[:w, :w] = leads.square_selfenergy(w, t, e)
    g[w:, w:] = -np.conj(leads.square_selfenergy(w, t, e))

    assert_almost_equal(g, modes_se(h_cell, h_hop))


def test_singular_h_and_t():
    h = 0.1 * np.identity(6)
    t = np.eye(6, 6, 4)
    sigma = modes_se(h, t)
    sigma_should_be = np.zeros((6,6))
    sigma_should_be[4, 4] = sigma_should_be[5, 5] = -10
    assert_almost_equal(sigma, sigma_should_be)


def test_modes():
    h, t = .3, .7
    k = np.arccos(-h / (2 * t))
    v = 2 * t * np.sin(k)
    prop, stab = leads.modes(np.array([[h]]), np.array([[t]]))
    assert stab.nmodes == 1
    assert stab.sqrt_hop is None
    np.testing.assert_almost_equal(prop.velocities, [-v, v])
    np.testing.assert_almost_equal(prop.momenta, [k, -k])
    # Test for normalization by current.
    np.testing.assert_almost_equal(
        2 * (stab.vecs[0] * stab.vecslmbdainv[0].conj()).imag, [1, -1])


def test_modes_bearded_ribbon():
    # Check if bearded graphene ribbons work.
    lat = kwant.lattice.honeycomb()
    sys = kwant.Builder(kwant.TranslationalSymmetry((1, 0)))
    sys[lat.shape((lambda pos: -20 < pos[1] < 20),
                  (0, 0))] = 0.3
    sys[lat.neighbors()] = -1
    sys = sys.finalized()
    h, t = sys.cell_hamiltonian(), sys.inter_cell_hopping()
    # The number of expected modes is calculated by plotting the dispersion.
    assert leads.modes(h, t)[1].nmodes == 8


def test_algorithm_equivalence():
    np.random.seed(400)
    n = 12
    h = np.random.randn(n, n) + 1j * np.random.randn(n, n)
    h += h.T.conj()
    t = np.random.randn(n, n) + 1j * np.random.randn(n, n)
    u, s, vh = np.linalg.svd(t)
    u, v = u * np.sqrt(s), vh.T.conj() * np.sqrt(s)
    prop_vecs = []
    evan_vecs = []
    algos = [None] + list(product(*(2 * [(True, False)])))
    for algo in algos:
        result = leads.modes(h, t, stabilization=algo)[1]

        vecs, vecslmbdainv = result.vecs, result.vecslmbdainv

        # Bring the calculated vectors to real space
        if algo is not None:
            vecs = np.dot(v, vecs)
            np.testing.assert_almost_equal(result.sqrt_hop, v)
        else:
            vecslmbdainv = np.dot(v.T.conj(), vecslmbdainv)
        full_vecs = np.r_[vecslmbdainv, vecs]

        prop_vecs.append(full_vecs[:, : 2 * result.nmodes])
        evan_vecs.append(full_vecs[:, 2 * result.nmodes :])

    msg = 'Stabilization {0} failed.'
    for vecs, algo in izip(prop_vecs, algos):
        # Propagating modes should have identical ordering, and only vary
        # By a phase
        np.testing.assert_allclose(np.abs(np.sum(vecs/prop_vecs[0],
                                                 axis=0)), vecs.shape[0],
                                   err_msg=msg.format(algo))

    for vecs, algo in izip(evan_vecs, algos):
        # Evanescent modes must span the same linear space.
        assert np.linalg.matrix_rank(np.c_[vecs, evan_vecs[0]], tol=1e-12) == \
               vecs.shape[1], msg.format(algo)


def test_for_all_evs_equal():
    """Test an 'ideal lead' which has all eigenvalues e^ik equal."""

    onsite = np.array([[0., 1.], [1., 0.]], dtype=complex)
    hopping = np.array([[0.0], [-1.0]], dtype=complex)

    modes = leads.modes(onsite, hopping)[1]

    assert modes.vecs.shape == (1, 2)
    assert modes.vecslmbdainv.shape == (1, 2)
    assert modes.nmodes == 1


def test_dtype_linsys():
    """Test that setup_linsys stays in real arithmetics when possible."""
    h_cell = np.array([[2.0, -1.0], [-1.0, 2.0]], dtype=np.float64)
    h_hop = np.array([[0.0],[-1.0]], dtype=np.float64)

    lsys = kwant.physics.leads.setup_linsys(h_cell - 0.3*np.eye(2),
                                            h_hop)
    assert lsys.eigenproblem[0].dtype == np.float64

    lsys = kwant.physics.leads.setup_linsys(h_cell.astype(np.complex128) 
                                            - 0.3*np.eye(2),
                                            h_hop.astype(np.complex128))
    assert lsys.eigenproblem[0].dtype == np.float64
    
    # energy=1 is an eigenstate of the isolated cell Hamiltonian,
    # i.e. a complex self-energy stabilization is necessary
    lsys = kwant.physics.leads.setup_linsys(h_cell - 1*np.eye(2),
                                            h_hop)
    assert lsys.eigenproblem[0].dtype == np.complex128

    # with complex input, output must be complex, too
    h_hop = np.array([[0.0],[-1.0 + 0.1j]], dtype=np.complex128)
    lsys = kwant.physics.leads.setup_linsys(h_cell - 0.3*np.eye(2),
                                            h_hop)
    assert lsys.eigenproblem[0].dtype == np.complex128



import square
from nose.tools import assert_equal, assert_raises
from numpy.testing import assert_almost_equal

def test_nodeid_to_from_pos():
    s = square.System((3, 4), 1)
    assert_raises(StandardError, s.nodeid_from_pos, (0, -2))
    assert_raises(StandardError, s.nodeid_from_pos, (-1, 3))
    assert_raises(StandardError, s.nodeid_from_pos, (3, 1))
    assert_raises(StandardError, s.pos_from_nodeid, -1)
    assert_raises(StandardError, s.pos_from_nodeid, 12)
    assert_equal(s.nodeid_from_pos((0, 0)), 0)
    assert_equal(s.nodeid_from_pos(s.pos_from_nodeid(7)), 7)
    assert_equal(s.pos_from_nodeid(s.nodeid_from_pos((2, 3))), (2, 3))

def test_hamiltonian():
    sys = square.System((4, 5), 1)
    for i in xrange(sys.graph.num_nodes):
        shape = sys.hamiltonian(i, i).shape
        assert_equal(len(shape), 2)
        assert_equal(shape[0], 1)
        for j in sys.graph.out_neighbors(i):
            m = sys.hamiltonian(i, j)
            shape = m.shape
            m_herm = m.T.conj()
            assert_almost_equal(m, m_herm)
            assert_almost_equal(m_herm, sys.hamiltonian(j, i))

def test_selfenergy():
    sys = square.System((2, 4), 1)
    for lead in xrange(len(sys.lead_interfaces)):
        se = sys.leads[lead].selfenergy(0)
        assert_equal(len(se.shape), 2)
        assert_equal(se.shape[0], se.shape[1])
        assert_equal(se.shape[0], len(sys.lead_interfaces[lead]))



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

from numpy.testing import assert_array_almost_equal
import kwant
from math import pi, cos

def test_band_energies(N=5):
    sys = kwant.Builder(kwant.TranslationalSymmetry((-1, 0)))
    lat = kwant.lattice.square()
    sys[[lat(0, 0), lat(0, 1)]] = 3
    sys[lat(0, 1), lat(0, 0)] = -1
    sys[((lat(1, y), lat(0, y)) for y in range(2))] = -1

    band_energies = kwant.physics.Bands(sys.finalized())
    for i in range(-N, N):
        k = i * pi / N
        energies = band_energies(k)
        assert_array_almost_equal(sorted(energies),
                                  sorted([2 - 2 * cos(k), 4 - 2 * cos(k)]))



# Tutorial 2.5. Beyond square lattices: graphene
# ==============================================
#
# Physics background
# ------------------
#  Transport through a graphene quantum dot with a pn-junction
#
# Kwant features highlighted
# --------------------------
#  - Application of all the aspects of tutorials 1-3 to a more complicated
#    lattice, namely graphene

from __future__ import division  # so that 1/2 == 0.5, and not 0
from math import pi, sqrt, tanh

import kwant

# For computing eigenvalues
import scipy.sparse.linalg as sla

# For plotting
from matplotlib import pyplot


# Define the graphene lattice
sin_30, cos_30 = (1 / 2, sqrt(3) / 2)
#HIDDEN_BEGIN_hnla
graphene = kwant.lattice.general([(1, 0), (sin_30, cos_30)],
                                 [(0, 0), (0, 1 / sqrt(3))])
a, b = graphene.sublattices
#HIDDEN_END_hnla


#HIDDEN_BEGIN_shzy
def make_system(r=10, w=2.0, pot=0.1):

    #### Define the scattering region. ####
    # circular scattering region
    def circle(pos):
        x, y = pos
        return x ** 2 + y ** 2 < r ** 2

    sys = kwant.Builder()

    # w: width and pot: potential maximum of the p-n junction
    def potential(site):
        (x, y) = site.pos
        d = y * cos_30 + x * sin_30
        return pot * tanh(d / w)

    sys[graphene.shape(circle, (0, 0))] = potential
#HIDDEN_END_shzy

    # specify the hoppings of the graphene lattice in the
    # format expected by builder.HoppingKind
#HIDDEN_BEGIN_hsmc
    hoppings = (((0, 0), a, b), ((0, 1), a, b), ((-1, 1), a, b))
#HIDDEN_END_hsmc
#HIDDEN_BEGIN_bfwb
    sys[[kwant.builder.HoppingKind(*hopping) for hopping in hoppings]] = -1
#HIDDEN_END_bfwb

    # Modify the scattering region
#HIDDEN_BEGIN_efut
    del sys[a(0, 0)]
    sys[a(-2, 1), b(2, 2)] = -1
#HIDDEN_END_efut

    #### Define the leads. ####
#HIDDEN_BEGIN_aakh
    # left lead
    sym0 = kwant.TranslationalSymmetry(graphene.vec((-1, 0)))

    def lead0_shape(pos):
        x, y = pos
        return (-0.4 * r < y < 0.4 * r)

    lead0 = kwant.Builder(sym0)
    lead0[graphene.shape(lead0_shape, (0, 0))] = -pot
    lead0[[kwant.builder.HoppingKind(*hopping) for hopping in hoppings]] = -1

    # The second lead, going to the top right
    sym1 = kwant.TranslationalSymmetry(graphene.vec((0, 1)))

    def lead1_shape(pos):
        v = pos[1] * sin_30 - pos[0] * cos_30
        return (-0.4 * r < v < 0.4 * r)

    lead1 = kwant.Builder(sym1)
    lead1[graphene.shape(lead1_shape, (0, 0))] = pot
    lead1[[kwant.builder.HoppingKind(*hopping) for hopping in hoppings]] = -1
#HIDDEN_END_aakh

#HIDDEN_BEGIN_kmmw
    return sys, [lead0, lead1]
#HIDDEN_END_kmmw


#HIDDEN_BEGIN_zydk
def compute_evs(sys):
    # Compute some eigenvalues of the closed system
    sparse_mat = sys.hamiltonian_submatrix(sparse=True)

    evs = sla.eigs(sparse_mat, 2)[0]
    print evs.real
#HIDDEN_END_zydk


def plot_conductance(sys, energies):
    # Compute transmission as a function of energy
    data = []
    for energy in energies:
        smatrix = kwant.smatrix(sys, energy)
        data.append(smatrix.transmission(0, 1))

    pyplot.figure()
    pyplot.plot(energies, data)
    pyplot.xlabel("energy [t]")
    pyplot.ylabel("conductance [e^2/h]")
    pyplot.show()


def plot_bandstructure(flead, momenta):
    bands = kwant.physics.Bands(flead)
    energies = [bands(k) for k in momenta]

    pyplot.figure()
    pyplot.plot(momenta, energies)
    pyplot.xlabel("momentum [(lattice constant)^-1]")
    pyplot.ylabel("energy [t]")
    pyplot.show()


#HIDDEN The part of the following code block which begins with family_colors
#HIDDEN is included verbatim in the tutorial text because nested code examples
#HIDDEN are not supported.  Remember to update the tutorial text when you
#HIDDEN modify this block.
#HIDDEN_BEGIN_itkk
def main():
    pot = 0.1
    sys, leads = make_system(pot=pot)

    # To highlight the two sublattices of graphene, we plot one with
    # a filled, and the other one with an open circle:
    def family_colors(site):
        return 0 if site.family == a else 1

    # Plot the closed system without leads.
    kwant.plot(sys, site_color=family_colors, site_lw=0.1, colorbar=False)
#HIDDEN_END_itkk

    # Compute some eigenvalues.
#HIDDEN_BEGIN_jmbi
    compute_evs(sys.finalized())
#HIDDEN_END_jmbi

    # Attach the leads to the system.
    for lead in leads:
        sys.attach_lead(lead)

    # Then, plot the system with leads.
    kwant.plot(sys, site_color=family_colors, site_lw=0.1,
               lead_site_lw=0, colorbar=False)

    # Finalize the system.
    sys = sys.finalized()

    # Compute the band structure of lead 0.
    momenta = [-pi + 0.02 * pi * i for i in xrange(101)]
    plot_bandstructure(sys.leads[0], momenta)

    # Plot conductance.
    energies = [-2 * pot + 4. / 50. * pot * i for i in xrange(51)]
    plot_conductance(sys, energies)


# Call the main function if the script gets executed (as opposed to imported).
# See <http://docs.python.org/library/__main__.html>.
if __name__ == '__main__':
    main()



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

from __future__ import division

__all__ = ['Builder', 'Site', 'SiteFamily', 'SimpleSiteFamily', 'Symmetry',
           'HoppingKind', 'Lead', 'BuilderLead', 'SelfEnergyLead', 'ModesLead']

import abc
import sys
import warnings
import operator
from itertools import izip, islice, chain
import tinyarray as ta
import numpy as np
from . import system, graph, physics


################ Sites and site families

class Site(tuple):
    """A site, member of a `SiteFamily`.

    Sites are the vertices of the graph which describes the tight binding
    system in a `Builder`.

    A site is uniquely identified by its family and its tag.

    Parameters
    ----------
    family : an instance of `SiteFamily`
        The 'type' of the site.
    tag : a hashable python object
        The unique identifier of the site within the site family, typically a
        vector of integers.

    Raises
    ------
    ValueError
        If `tag` is not a proper tag for `family`.

    Notes
    -----
    For convenience, ``family(*tag)`` can be used instead of ``Site(family,
    tag)`` to create a site.

    The parameters of the constructor (see above) are stored as instance
    variables under the same names.  Given a site ``site``, common things to
    query are thus ``site.family``, ``site.tag``, and ``site.pos``.
    """
    __slots__ = ()

    family = property(operator.itemgetter(0),
                      doc="The site family to which the site belongs.")
    tag = property(operator.itemgetter(1), doc="The tag of the site.")


    def __new__(cls, family, tag, _i_know_what_i_do=False):
        if _i_know_what_i_do:
            return tuple.__new__(cls, (family, tag))
        try:
            tag = family.normalize_tag(tag)
        except (TypeError, ValueError):
            t, v, tb = sys.exc_info()
            msg = 'Tag {0} is not allowed for site family {1}: {2}'
            raise t(msg.format(repr(tag), repr(family), v))
        return tuple.__new__(cls, (family, tag))

    def __repr__(self):
        return 'Site({0}, {1})'.format(repr(self.family), repr(self.tag))

    def __str__(self):
        return '{1} of {0}'.format(str(self.family), str(self.tag))

    @property
    def pos(self):
        """Real space position of the site."""
        return self.family.pos(self.tag)


class SiteFamily(object):
    """
    Abstract base class for site families.

    A site family is a 'type' of sites.  Site families must be immutable and
    fully defined by their initial arguments.  They must inherit from this
    abstract base class and call its __init__ function providing it with two
    arguments: a canonical representation and a name.  The canonical
    representation will be returned as the objects representation and must
    uniquely identify the site family instance.  The name is a string used to
    distinguish otherwise identical site families.  It may be empty.

    All site families must define the method `normalize_tag` which brings a tag
    to the standard format for this site family.

    Site families which are intended for use with plotting should also provide a
    method `pos(tag)`, which returns a vector with real-space coordinates of
    the site belonging to this family with a given tag.
    """
    __metaclass__ = abc.ABCMeta

    def __init__(self, canonical_repr, name):
        self.canonical_repr = canonical_repr
        self.hash = hash(canonical_repr)
        self.name = name

    def __repr__(self):
        return self.canonical_repr

    def __str__(self):
        return '{0} object {1}'.format(self.__class__, self.name)

    def __hash__(self):
        return self.hash

    def __eq__(self, other):
        try:
            return self.canonical_repr == other.canonical_repr
        except AttributeError:
            return False

    def __ne__(self, other):
        try:
            return self.canonical_repr != other.canonical_repr
        except AttributeError:
            return False

    @abc.abstractmethod
    def normalize_tag(self, tag):
        """Return a normalized version of the tag.

        Raises TypeError or ValueError if the tag is not acceptable.
        """
        pass

    def __call__(self, *tag):
        """
        A convenience function.

        This function allows to write sg(1, 2) instead of Site(sg, (1, 2)).
        """
        # Catch a likely and difficult to find mistake.
        if tag and isinstance(tag[0], tuple):
            raise ValueError('Use site_family(1, 2) instead of '
                             'site_family((1, 2))!')
        return Site(self, tag)


class SimpleSiteFamily(SiteFamily):
    """A site family used as an example and for testing.

    A family of sites tagged by any python objects where object satisfied
    condition ``object == eval(repr(object))``.

    It exists to provide a basic site family that can be used for testing the
    builder module without other dependencies.  It can be also used to tag
    sites with non-numeric objects like strings should this every be useful.

    Due to its low storage efficiency for numbers it is not recommended to use
    `SimpleSiteFamily` when `kwant.lattice.Monatomic` would also work.
    """

    def __init__(self, name=None):
        canonical_repr = '{0}({1})'.format(self.__class__, repr(name))
        super(SimpleSiteFamily, self).__init__(canonical_repr, name)

    def normalize_tag(self, tag):
        tag = tuple(tag)
        try:
            if eval(repr(tag)) != tag:
                raise RuntimeError()
        except:
            raise TypeError('It must be possible to recreate the tag from '
                            'its representation.')
        return tag


################ Symmetries

class Symmetry(object):
    """Abstract base class for spatial symmetries.

    Many physical systems possess a discrete spatial symmetry, which results in
    special properties of these systems.  This class is the standard way to
    describe discrete spatial symmetries in Kwant.  An instance of this class
    can be passed to a `Builder` instance at its creation.  The most important
    kind of symmetry is translational symmetry, used to define scattering
    leads.

    Each symmetry has a fundamental domain -- a set of sites and hoppings,
    generating all the possible sites and hoppings upon action of symmetry
    group elements.  A class derived from `Symmetry` has to implement mapping
    of any site or hopping into the fundamental domain, applying a symmetry
    group element to a site or a hopping, and a method `which` to determine the
    group element bringing some site from the fundamental domain to the
    requested one.  Additionally, it has to have a property `num_directions`
    returning the number of independent symmetry group generators (number of
    elementary periods for translational symmetry).

    A ``ValueError`` must be raised by the symmetry class whenever a symmetry
    is used together with sites whose site family is not compatible with it.  A
    typical example of this is when the vector defining a translational
    symmetry is not a lattice vector.
    """
    __metaclass__ = abc.ABCMeta

    @abc.abstractproperty
    def num_directions(self):
        """Number of elementary periods of the symmetry."""
        pass

    @abc.abstractmethod
    def which(self, site):
        """Calculate the domain of the site.

        Return the group element whose action on a certain site from the
        fundamental domain will result in the given `site`.
        """
        pass

    @abc.abstractmethod
    def act(self, element, a, b=None):
        """Act with a symmetry group element on a site or hopping."""
        pass

    def to_fd(self, a, b=None):
        """Map a site or hopping to the fundamental domain.

        If `b` is None, return a site equivalent to `a` within the fundamental
        domain.  Otherwise, return a hopping equivalent to `(a, b)` but where
        the first element belongs to the fundamental domain.

        This default implementation works but may be not efficient.
        """
        return self.act(-self.which(a), a, b)

    def in_fd(self, site):
        """Tell whether `site` lies within the fundamental domain."""
        for d in self.which(site):
            if d != 0:
                return False
        return True


class NoSymmetry(Symmetry):
    """A symmetry with a trivial symmetry group."""

    def __eq__(self, other):
        return isinstance(other, NoSymmetry)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return 'NoSymmetry()'

    @property
    def num_directions(self):
        return 0

    _empty_array = ta.array((), int)

    def which(self, site):
        return self._empty_array

    def act(self, element, a, b=None):
        if element:
            raise ValueError('`element` must be empty for NoSymmetry.')
        return a if b is None else (a, b)

    def to_fd(self, a, b=None):
        return a if b is None else (a, b)

    def in_fd(self, site):
        return True


################ Hopping kinds

class HoppingKind(object):
    """A pattern for matching hoppings.

    A hopping ``(a, b)`` matches precisely when the site family of ``a`` equals
    `family_a` and that of ``b`` equals `family_b` and ``(a.tag - b.tag)`` is
    equal to `delta`.  In other words, the matching hoppings have the form:
    ``(family_a(x + delta), family_b(x))``

    Parameters
    ----------
    delta : Sequence of integers
        The sequence is interpreted as a vector with integer elements.
    family_a : `~kwant.builder.SiteFamily`
    family_b : `~kwant.builder.SiteFamily` or ``None`` (default)
        The default value means: use the same family as `family_a`.

    Notes
    -----
    A ``HoppingKind`` is a callable object: When called with a
    `~kwant.builder.Builder` as sole argument, an instance of this class will
    return an iterator over all possible matching hoppings whose sites are
    already present in the system.  The hoppings do *not* have to be already
    present in the system.  For example::

        kind = kwant.builder.HoppingKind((1, 0), lat)
        sys[kind(sys)] = 1

    Because a `~kwant.builder.Builder` can be indexed with functions or
    iterables of functions, ``HoppingKind`` instances (or any non-tuple
    iterables of them, e.g. a list) can be used directly as "wildcards" when
    setting or deleting hoppings::

        kinds = [kwant.builder.HoppingKind(v, lat) for v in [(1, 0), (0, 1)]]
        sys[kinds] = 1
    """
    __slots__ = ('delta', 'family_a', 'family_b')

    def __init__(self, delta, family_a, family_b=None):
        self.delta = ta.array(delta, int)
        self.family_a = family_a
        self.family_b = family_b if family_b is not None else family_a

    def __call__(self, builder):
        delta = self.delta
        family_a = self.family_a
        family_b = self.family_b
        H = builder.H
        symtofd = builder.symmetry.to_fd

        for a in H:
            if a.family != family_a:
                continue
            b = Site(family_b, a.tag - delta, True)
            if symtofd(b) in H:
                yield a, b

    def __repr__(self):
        return '{0}({1}, {2}{3})'.format(
            self.__class__.__name__, repr(tuple(self.delta)),
            repr(self.family_a),
            ', ' + repr(self.family_b) if self.family_a != self.family_b else '')


################ Support for Hermitian conjugation

def herm_conj(value):
    """
    Calculate the hermitian conjugate of a python object.

    If the object is neither a complex number nor a matrix, the original value
    is returned.  In the context of this module, this is the correct behavior
    for functions.
    """
    if hasattr(value, 'conjugate'):
        value = value.conjugate()
        if hasattr(value, 'transpose'):
            value = value.transpose()
    return value


class HermConjOfFunc(object):
    """Proxy returning the hermitian conjugate of the original result."""
    __slots__ = ('function')

    def __init__(self, function):
        self.function = function

    def __call__(self, i, j):
        return herm_conj(self.function(j, i))


################ Leads

class Lead(object):
    """Abstract base class for leads that can be attached to a `Builder`.

    Attributes
    ----------
    interface : sequence of sites
    """
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def finalized():
        """Return a finalized version of the lead.

        Returns
        -------
        finalized_lead

        Notes
        -----
        The finalized lead must be an object that can be used as a lead in a
        `kwant.system.FiniteSystem`.  It could be an instance of
        `kwant.system.InfiniteSystem` for example.

        The order of sites for the finalized lead must be the one specified in
        `interface`.
        """
        pass


class BuilderLead(Lead):
    """A lead made from a `Builder` with a spatial symmetry.

    Parameters
    ----------
    builder : `Builder`
        The tight-binding system of a lead. It has to possess appropriate
        symmetry, and it may not contain hoppings between further than
        neighboring images of the fundamental domain.
    interface : sequence of `Site` instances
        Sequence of sites in the scattering region to which the lead is
        attached.

    Notes
    -----
    The hopping from the scattering region to the lead is assumed to be equal to
    the hopping from a lead unit cell to the next one in the direction of the
    symmetry vector (i.e. the lead is 'leaving' the system and starts with a
    hopping).

    The given order of interface sites is preserved throughout finalization.

    Every system has an attribute `leads`, which stores a list of
    `BuilderLead` objects with all the information about the leads that are
    attached.
    """
    def __init__(self, builder, interface):
        self.builder = builder
        self.interface = tuple(interface)

    def finalized(self):
        """Return a `kwant.system.InfiniteSystem` corresponding to the
        compressed lead.

        The order of interface sites is kept during finalization.
        """
        return self.builder._finalized_infinite(self.interface)


class SelfEnergyLead(Lead):
    """A general lead defined by its self energy.

    Parameters
    ----------
    selfenergy_func : function
        Function which returns the self energy matrix for the interface sites
        given the energy and optionally a list of extra arguments.
    interface : sequence of `Site` instances
    """
    def __init__(self, selfenergy_func, interface):
        self._selfenergy_func = selfenergy_func
        self.interface = tuple(interface)

    def finalized(self):
        """Trivial finalization: the object is returned itself."""
        return self

    def selfenergy(self, energy, args=()):
        return self._selfenergy_func(energy, args)


class ModesLead(Lead):
    """A general lead defined by its modes wave functions.

    Parameters
    ----------
    modes_func : function
        Function which returns the modes of the lead as a tuple of
        `~kwant.physics.PropagatingModes` and `~kwant.physics.StabilizedModes`
        given the energy and optionally a list of extra arguments.
    interface :
        sequence of `Site` instances

    """
    def __init__(self, modes_func, interface):
        self.modes_func = modes_func
        self.interface = tuple(interface)

    def finalized(self):
        """Trivial finalization: the object is returned itself."""
        return self

    def modes(self, energy, args=()):
        return self.modes_func(energy, args)

    def selfenergy(self, energy, args=()):
        propagating, stabilized = self.modes(energy, args)
        return stabilized.selfenergy()



################ Builder class

# A marker, meaning for hopping (i, j): this value is given by the Hermitian
# conjugate the value of the hopping (j, i).  Used by Builder and System.
Other = type('Other', (object,), {'__repr__': lambda s: 'Other'})()


def edges(seq):
    # izip, when given the same iterator twice, turns a sequence into a
    # sequence of pairs.
    seq_iter = iter(seq)
    result = izip(seq_iter, seq_iter)
    next(result)                # Skip the special loop edge.
    return result


class Builder(object):
    """A tight binding system defined on a graph.

    This is one of the central types in Kwant.  It is used to construct tight
    binding systems in a flexible way.

    The nodes of the graph are `Site` instances.  The edges, i.e. the hoppings,
    are pairs (2-tuples) of sites.  Each node and each edge has a value
    associated with it.  The values associated with nodes are interpreted as
    on-site Hamiltonians, the ones associated with edges as hopping integrals.

    To make the graph accessible in a way that is natural within the Python
    language it is exposed as a *mapping* (much like a built-in Python
    dictionary).  Keys are sites or hoppings.  Values are 2d arrays
    (e.g. NumPy or Tinyarray) or numbers (interpreted as 1 by 1 matrices).

    Parameters
    ----------
    symmetry : `Symmetry` or `None`
        The symmetry of the system.

    Notes
    -----
    Values can be also functions that receive the site or the hopping (passed
    to the function as two sites) and possibly additional arguments and are
    expected to return a valid value.  This allows to define systems quickly,
    to modify them without reconstructing, and to save memory for many-orbital
    models.

    In addition to simple keys (single sites and hoppings) more powerful keys
    are possible as well that allow to manipulate multiple sites/hoppings in a
    single operation.  Such keys are internally expanded into a sequence of
    simple keys by using the method `Builder.expand`.  For example,
    ``sys[general_key] = value`` is equivalent to ::

        for simple_key in sys.expand(general_key):
            sys[simple_key] = value

    Builder instances automatically ensure that every hopping is Hermitian, so
    that if ``builder[a, b]`` has been set, there is no need to set
    ``builder[b, a]``.

    Builder instances can be made to automatically respect a `Symmetry` that is
    passed to them during creation.  The behavior of builders with a symmetry
    is slightly more sophisticated.  First of all, it is implicitly assumed
    throughout Kwant that **every** function assigned as a value to a builder
    with a symmetry possesses the same symmetry.  Secondly, all keys are mapped
    to the fundamental domain of the symmetry before storing them.  This may
    produce confusing results when neighbors of a site are queried.

    The method `attach_lead` *works* only if the sites affected by them have
    tags which are sequences of integers.  It *makes sense* only when these
    sites live on a regular lattice, like the ones provided by `kwant.lattice`.

    ``builder0 += builder1`` adds all the sites, hoppings, and leads of
    ``builder1`` to ``builder0``.  Sites and hoppings present in both systems
    are overwritten by those in ``builder1``.  The leads of ``builder1`` are
    appended to the leads of the system being extended.

    .. warning::

        If functions are used to set values in a builder with a symmetry, then
        they must satisfy the same symmetry.  There is (currently) no check and
        wrong results will be the consequence of a misbehaving function.

    Examples
    --------
    Define a site.

    >>> builder[site] = value

    Print the value of a site.

    >>> print builder[site]

    Define a hopping.

    >>> builder[site1, site2] = value

    Delete a site.

    >>> del builder[site3]

    """

    def __init__(self, symmetry=None):
        if symmetry is None:
            symmetry = NoSymmetry()
        self.symmetry = symmetry
        self.leads = []
        self.H = {}

    #### Note on H ####
    #
    # This dictionary stores a directed graph optimized for efficient querying
    # and modification.  The nodes are instances of `Site`.
    #
    # Each edge, specified by a ``(tail, head)`` pair of nodes, holds an object
    # as a value.  Likewise, each tail which occurs in the graph also holds a
    # value.  (Nodes which only occur as heads are not required to have
    # values.)
    #
    # For a given `tail` site, H[tail] is a list alternately storing
    # heads and values.  (The heads occupy even locations followed by the
    # values at odd locations.)  Each pair of entries thus describes a single
    # directed edge of the graph.
    #
    # The first pair of entries in each list is special: it always
    # corresponds to a loop edge.  (The head is equal to the tail.)  This
    # special edge has two purposes: It is used to store the value
    # associated with the tail node itself, and it is necessary for the
    # method getkey_tail which helps to conserve memory by storing equal
    # node label only once.

    def _get_edge(self, tail, head):
        for h, value in edges(self.H[tail]):
            if h == head:
                return value
        raise ValueError(tail, head)

    def _set_edge(self, tail, head, value):
        hvhv = self.H[tail]
        heads = hvhv[2::2]
        if head in heads:
            i = 2 + 2 * heads.index(head)
            hvhv[i] = head
            hvhv[i + 1] = value
        else:
            hvhv.append(head)
            hvhv.append(value)

    def _del_edge(self, tail, head):
        hvhv = self.H[tail]
        heads = hvhv[2::2]
        i = 2 + 2 * heads.index(head)
        del hvhv[i : i + 2]

    def _out_neighbors(self, tail):
        hvhv = self.H.get(tail, ())
        return islice(hvhv, 2, None, 2)

    def _out_degree(self, tail):
        hvhv = self.H.get(tail, ())
        return len(hvhv) // 2 - 1

    # TODO: write a test for this method.
    def reversed(self):
        """Return a shallow copy of the builder with the symmetry reversed.

        This method can be used to attach the same infinite system as lead from
        two opposite sides.  It requires a builder to which an infinite
        symmetry is associated.
        """
        result = object.__new__(Builder)
        result.symmetry = self.symmetry.reversed()
        if self.leads:
            raise ValueError('System to be reversed may not have leads.')
        result.leads = []
        result.H = self.H
        return result

    def __nonzero__(self):
        return bool(self.H)

    def expand(self, key):
        """
        Expand a general key into an iterator over simple keys.

        Parameters
        ----------
        key : builder key (see notes)
            The key to be expanded

        Notes
        -----
        Keys are (recursively):
            * Simple keys: sites or 2-tuples of sites (=hoppings).
            * Any (non-tuple) iterable of keys, e.g. a list or a generator
              expression.
            * Any function that returns a key when passed a builder as sole
              argument, e.g. a `HoppingKind` instance or the function returned
              by `~kwant.lattice.Polyatomic.shape`.

        This method is internally used to expand the keys when getting or
        deleting items of a builder (i.e. ``sys[key] = value`` or ``del
        sys[key]``).

        """
        itr = iter((key,))
        iter_stack = [None]
        while iter_stack:
            for key in itr:
                while callable(key):
                    key = key(self)
                if isinstance(key, Site) or isinstance(key, tuple):
                    yield key
                else:
                    iter_stack.append(itr)
                    itr = iter(key)
                    break
            else:
                itr = iter_stack.pop()

    def _get_site(self, site):
        site = self.symmetry.to_fd(site)
        try:
            return self.H[site][1]
        except KeyError:
            raise KeyError(site)

    def _get_hopping(self, hopping):
        sym = self.symmetry
        try:
            a, b = hopping
        except:
            raise KeyError(hopping)
        try:
            a, b = sym.to_fd(a, b)
            value = self._get_edge(a, b)
        except ValueError:
            raise KeyError(hopping)
        if value is Other:
            if not sym.in_fd(b):
                b, a = sym.to_fd(b, a)
                assert not sym.in_fd(a)
            value = self._get_edge(b, a)
            if callable(value):
                assert not isinstance(value, HermConjOfFunc)
                value = HermConjOfFunc(value)
            else:
                value = herm_conj(value)
        return value

    def __getitem__(self, key):
        """Get the value of a single site or hopping."""
        if isinstance(key, Site):
            return self._get_site(key)
        elif isinstance(key, tuple):
            return self._get_hopping(key)
        else:
            raise KeyError(key)

    def __contains__(self, key):
        """Tell whether the system contains a site or hopping."""
        if isinstance(key, Site):
            site = self.symmetry.to_fd(key)
            return site in self.H
        elif isinstance(key, tuple):
            a, b = key
            a, b = self.symmetry.to_fd(a, b)
            hvhv = self.H.get(a, ())
            return b in islice(hvhv, 2, None, 2)
        else:
            raise KeyError(key)

    def _set_site(self, site, value):
        """Set a single site."""
        site = self.symmetry.to_fd(site)
        hvhv = self.H.setdefault(site, [])
        if hvhv:
            hvhv[1] = value
        else:
            hvhv[:] = [site, value]

    def _set_hopping(self, hopping, value):
        """Set a single hopping."""
        # Avoid nested HermConjOfFunc instances.
        try:
            a, b = hopping
        except:
            raise KeyError(hopping)
        if a == b:
            raise KeyError(hopping)
        if isinstance(value, HermConjOfFunc):
            a, b = b, a
            value = value.function

        sym = self.symmetry

        try:
            a, b = sym.to_fd(a, b)
            if sym.in_fd(b):
                # These two following lines make sure we do not waste space by
                # storing different instances of identical sites.  They also
                # verify that sites a and b already belong to the system.
                a = self.H[a][0]                 # Might fail.
                b = self.H[b][0]                 # Might fail.
                self._set_edge(a, b, value)      # Will work.
                self._set_edge(b, a, Other)      # Will work.
            else:
                b2, a2 = sym.to_fd(b, a)
                if b2 not in self.H:
                    raise KeyError()
                assert not sym.in_fd(a2)
                self._set_edge(a, b, value)      # Might fail.
                self._set_edge(b2, a2, Other)    # Will work.
        except KeyError:
            raise KeyError(hopping)

    def __setitem__(self, key, value):
        """Set a single site/hopping or a bunch of them."""
        # TODO: Once we can take Python 3 for granted, get rid of the if-clause
        # inside the loop by defining a special func that rebinds itself upon
        # the first call.
        func = None
        for sh in self.expand(key):
            if func is None:
                func = self._set_site if isinstance(sh, Site) \
                    else self._set_hopping
            func(sh, value)

    def _del_site(self, site):
        """Delete a single site and all associated hoppings."""
        tfd = self.symmetry.to_fd
        site = tfd(site)
        try:
            for neighbor in self._out_neighbors(site):
                if neighbor in self.H:
                    self._del_edge(neighbor, site)
                else:
                    assert not self.symmetry.in_fd(neighbor)
                    a, b = tfd(neighbor, site)
                    self._del_edge(a, b)
        except ValueError:
            raise KeyError(site)
        del self.H[site]

    def _del_hopping(self, hopping):
        """Delete a single hopping."""
        sym = self.symmetry

        try:
            a, b = hopping
        except:
            raise KeyError(hopping)
        try:
            a, b = sym.to_fd(a, b)
            if sym.in_fd(b):
                self._del_edge(a, b)
                self._del_edge(b, a)
            else:
                self._del_edge(a, b)
                b, a = sym.to_fd(b, a)
                assert not sym.in_fd(a)
                self._del_edge(b, a)
        except ValueError:
            raise KeyError(hopping)

    def __delitem__(self, key):
        """Delete a single site/hopping or bunch of them."""
        # TODO: Once we can take Python 3 for granted, get rid of the if-clause
        # inside the loop by defining a special func that rebinds itself upon
        # the first call.
        func = None
        for sh in self.expand(key):
            if func is None:
                func = self._del_site if isinstance(sh, Site) \
                    else self._del_hopping
            func(sh)

    def eradicate_dangling(self):
        """Keep deleting dangling sites until none are left."""
        sites = list(site for site in self.H
                     if self._out_degree(site) < 2)
        for site in sites:
            if site not in self.H:
                continue
            while site:
                neighbors = tuple(self._out_neighbors(site))
                if neighbors:
                    assert len(neighbors) == 1
                    neighbor = neighbors[0]
                    self._del_edge(neighbor, site)
                    if self._out_degree(neighbor) > 1:
                        neighbor = False
                else:
                    neighbor = False
                del self.H[site]
                site = neighbor

    def __iter__(self):
        """Return an iterator over all sites and hoppings."""
        return chain(self.H, self.hoppings())

    def sites(self):
        """Return a read-only set over all sites.

        The sites that are returned belong to the fundamental domain of the
        `Builder` symmetry, and are not necessarily the ones that were set
        initially (but always the equivalent ones).
        """
        try:
            return self.H.viewkeys()
        except AttributeError:
            return frozenset(self.H)

    def site_value_pairs(self):
        """Return an iterator over all (site, value) pairs."""
        for site, hvhv in self.H.iteritems():
            yield site, hvhv[1]

    def hoppings(self):
        """Return an iterator over all Builder hoppings.

        The hoppings that are returned belong to the fundamental domain of the
        `Builder` symmetry, and are not necessarily the ones that were set
        initially (but always the equivalent ones).
        """
        for tail, hvhv in self.H.iteritems():
            for head, value in edges(hvhv):
                if value is Other:
                    continue
                yield (tail, head)

    def hopping_value_pairs(self):
        """Return an iterator over all (hopping, value) pairs."""
        for tail, hvhv in self.H.iteritems():
            for head, value in edges(hvhv):
                if value is Other:
                    continue
                yield (tail, head), value

    def dangling(self):
        """Return an iterator over all dangling sites."""
        for site in self.H:
            if self._out_degree(site) < 2:
                yield site

    def degree(self, site):
        """Return the number of neighbors of a site."""
        site = self.symmetry.to_fd(site)
        return self._out_degree(site)

    def neighbors(self, site):
        """Return an iterator over all neighbors of a site."""
        a = self.symmetry.to_fd(site)
        return self._out_neighbors(a)

    def __iadd__(self, other_sys):
        for site, value in other_sys.site_value_pairs():
            self[site] = value
        for hop, value in other_sys.hopping_value_pairs():
            self[hop] = value
        self.leads.extend(other_sys.leads)
        return self

    def attach_lead(self, lead_builder, origin=None):
        """Attach a lead to the builder, possibly adding missing sites.

        Parameters
        ----------
        lead_builder : `Builder` with 1D translational symmetry
            Builder of the lead which has to be attached.
        origin : `Site`
            The site which should belong to a domain where the lead should
            begin. It is used to attach a lead inside the system, e.g. to an
            inner radius of a ring.

        Raises
        ------
        ValueError
            If `lead_builder` does not have proper symmetry, has hoppings with
            range of more than one lead unit cell, or if it is not completely
            interrupted by the system.

        Notes
        -----
        This method is not fool-proof, i.e. if it returns an error, there is
        no guarantee that the system stayed unaltered.
        """
        sym = lead_builder.symmetry
        H = lead_builder.H

        if sym.num_directions != 1:
            raise ValueError('Only builders with a 1D symmetry are allowed.')
        for hopping in lead_builder.hoppings():
            if not -1 <= sym.which(hopping[1])[0] <= 1:
                msg = 'Hopping {0} connects non-neighboring lead unit cells.' +\
                      'Only nearest-cell hoppings are allowed ' +\
                      '(consider increasing the lead period).'
                raise ValueError(msg.format(hopping))
        if not H:
            raise ValueError('Lead to be attached contains no sites.')

        # Check if site families of the lead are present in the system (catches
        # a common and a hard to find bug).
        families = set(site.family for site in H)
        lead_only_families = families.copy()
        for site in self.H:
            lead_only_families.discard(site.family)
            if not lead_only_families:
                break
        else:
            msg = 'Sites with site families {0} do not appear in the system, ' \
                'hence the system does not interrupt the lead.'
            raise ValueError(msg.format(tuple(lead_only_families)))

        all_doms = set()
        for site in self.H:
            if site.family not in families:
                continue
            ge = sym.which(site)
            if sym.act(-ge, site) in H:
                all_doms.add(ge[0])

        if origin is not None:
            orig_dom = sym.which(origin)[0]
            all_doms = [dom for dom in all_doms if dom <= orig_dom]
        if len(all_doms) == 0:
            raise ValueError('Builder does not intersect with the lead,'
                             ' this lead cannot be attached.')
        max_dom = max(all_doms)
        min_dom = min(all_doms)
        del all_doms

        interface = set()
        added = set()
        # Initialize flood-fill: create the outermost sites.
        for site in H:
            for neighbor in lead_builder.neighbors(site):
                neighbor = sym.act((max_dom + 1,), neighbor)
                if sym.which(neighbor)[0] == max_dom:
                    if neighbor not in self:
                        self[neighbor] = lead_builder[neighbor]
                        added.add(neighbor)
                    interface.add(neighbor)

        # Do flood-fill.
        covered = True
        while covered:
            covered = False
            added2 = set()
            for site in added:
                site_dom = sym.which(site)
                move = lambda x: sym.act(site_dom, x)
                for site_new in lead_builder.neighbors(site):
                    site_new = move(site_new)
                    new_dom = sym.which(site_new)[0]
                    if new_dom == max_dom + 1:
                        continue
                    elif new_dom < min_dom:
                        raise ValueError('Builder does not interrupt the lead,'
                                         ' this lead cannot be attached.')
                    if site_new not in self \
                       and sym.which(site_new)[0] != max_dom + 1:
                        self[site_new] = lead_builder[site_new]
                        added2.add(site_new)
                        covered = True
                    self[site_new, site] = lead_builder[site_new, site]
            added = added2

        self.leads.append(BuilderLead(lead_builder, tuple(interface)))
        return len(self.leads) - 1

    def finalized(self):
        """Return a finalized (=usable with solvers) copy of the system.

        Returns
        -------
        finalized_system : `kwant.system.FiniteSystem`
            If there is no symmetry.
        finalized_system : `kwant.system.InfiniteSystem`
            If a symmetry is present.

        Notes
        -----
        This method does not modify the Builder instance for which it is
        called.

        Attached leads are also finalized and will be present in the finalized
        system to be returned.

        Currently, only Builder instances without or with a 1D translational
        `Symmetry` can be finalized.
        """
        if self.symmetry.num_directions == 0:
            return self._finalized_finite()
        elif self.symmetry.num_directions == 1:
            return self._finalized_infinite()
        else:
            raise ValueError('Currently, only builders without or with a 1D '
                             'translational symmetry can be finalized.')

    def _finalized_finite(self):
        assert self.symmetry.num_directions == 0

        #### Make translation tables.
        sites = tuple(self.H)
        id_by_site = {}
        for site_id, site in enumerate(sites):
            id_by_site[site] = site_id

        #### Make graph.
        g = graph.Graph()
        g.num_nodes = len(sites)  # Some sites could not appear in any edge.
        for tail, hvhv in self.H.iteritems():
            for head in islice(hvhv, 2, None, 2):
                if tail == head:
                    continue
                g.add_edge(id_by_site[tail], id_by_site[head])
        g = g.compressed()

        #### Connect leads.
        finalized_leads = []
        lead_interfaces = []
        for lead_nr, lead in enumerate(self.leads):
            try:
                with warnings.catch_warnings(record=True) as ws:
                    warnings.simplefilter("always")
                    # The following line is the whole "payload" of the entire
                    # try-block.
                    finalized_leads.append(lead.finalized())
                for w in ws:
                    # Re-raise any warnings with an additional message and the
                    # proper stacklevel.
                    w = w.message
                    msg = 'When finalizing lead {0}:'.format(lead_nr)
                    warnings.warn(w.__class__(' '.join((msg,) + w.args)),
                                  stacklevel=3)
            except ValueError, e:
                # Re-raise the exception with an additional message.
                msg = 'Problem finalizing lead {0}:'.format(lead_nr)
                e.args = (' '.join((msg,) + e.args),)
                raise
            interface = [id_by_site[isite] for isite in lead.interface]
            lead_interfaces.append(np.array(interface))

        #### Assemble and return result.
        result = FiniteSystem()
        result.graph = g
        result.sites = sites
        result.leads = finalized_leads
        result.hoppings = [self._get_edge(sites[tail], sites[head])
                           for tail, head in g]
        result.onsite_hamiltonians = [self.H[site][1] for site in sites]
        result.lead_interfaces = lead_interfaces
        result.symmetry = self.symmetry
        return result

    def _finalized_infinite(self, interface_order=None):
        """
        Finalize this builder instance which has to have exactly a single
        symmetry direction.

        If interface_order is not set, the order of the interface sites in the
        finalized system will be arbitrary.  If interface_order is set to a
        sequence of interface sites, this order will be kept.
        """
        sym = self.symmetry
        assert sym.num_directions == 1

        #### For each site of the fundamental domain, determine whether it has
        #### neighbors in the previous domain or not.
        lsites_with = []       # Fund. domain sites with neighbors in prev. dom
        lsites_without = []    # Remaining sites of the fundamental domain
        for tail in self.H:    # Loop over all sites of the fund. domain.
            for head in self._out_neighbors(tail):
                fd = sym.which(head)[0]
                if fd == 1:
                    # Tail belongs to fund. domain, head to the next domain.
                    lsites_with.append(tail)
                    break
            else:
                # Tail is a fund. domain site not connected to prev. domain.
                lsites_without.append(tail)
        cell_size = len(lsites_with) + len(lsites_without)

        if not lsites_with:
            warnings.warn('Infinite system with disconnected cells.',
                          RuntimeWarning, stacklevel=3)

        ### Create list of sites and a lookup table
        minus_one = ta.array((-1,))
        plus_one = ta.array((1,))
        if interface_order is None:
            interface = [sym.act(minus_one, s) for s in lsites_with]
        else:
            lsites_with_set = set(lsites_with)
            lsites_with = []
            interface = []
            if interface_order:
                shift = ta.array((-sym.which(interface_order[0])[0] - 1,))
            for shifted_iface_site in interface_order:
                # Shift the interface domain before the fundamental domain.
                # That's the right place for the interface of a lead to be, but
                # the sites of interface_order might live in a different
                # domain.
                iface_site = sym.act(shift, shifted_iface_site)
                lsite = sym.act(plus_one, iface_site)

                try:
                    lsites_with_set.remove(lsite)
                except KeyError:
                    if (-sym.which(shifted_iface_site)[0] - 1,) != shift:
                        raise ValueError(
                            'The sites in interface_order do not all '
                            'belong to the same lead cell.')
                    else:
                        raise ValueError('A site in interface_order is not an '
                                         'interface site:\n' + str(iface_site))
                interface.append(iface_site)
                lsites_with.append(lsite)
            if lsites_with_set:
                raise ValueError(
                    'interface_order did not contain all interface sites.')
            del lsites_with_set

        sites = lsites_with + lsites_without + interface
        del lsites_with
        del lsites_without
        del interface
        id_by_site = {}
        for site_id, site in enumerate(sites):
            id_by_site[site] = site_id

        #### Make graph and extract onsite Hamiltonians.
        g = graph.Graph()
        g.num_nodes = len(sites)  # Some sites could not appear in any edge.
        onsite_hamiltonians = []
        for tail_id, tail in enumerate(sites[:cell_size]):
            onsite_hamiltonians.append(self.H[tail][1])
            for head in self._out_neighbors(tail):
                head_id = id_by_site.get(head)
                if head_id is None:
                    # Head belongs neither to the fundamental domain nor to the
                    # previous domain.  Check that it belongs to the next
                    # domain and ignore it otherwise as an edge corresponding
                    # to this one has been added already or will be added.
                    fd = sym.which(head)[0]
                    if fd != 1:
                        msg = 'Further-than-nearest-neighbor cells ' \
                              'are connected by hopping\n{0}.'
                        raise ValueError(msg.format((tail, head)))
                    continue
                if head_id >= cell_size:
                    # Head belongs to previous domain.  The edge added here
                    # correspond to one left out just above.
                    g.add_edge(head_id, tail_id)
                g.add_edge(tail_id, head_id)
        del id_by_site
        g = g.compressed()

        #### Extract hoppings.
        hoppings = []
        for tail_id, head_id in g:
            tail = sites[tail_id]
            head = sites[head_id]
            if tail_id >= cell_size:
                # The tail belongs to the previous domain.  Find the
                # corresponding hopping with the tail in the fund. domain.
                tail, head = sym.to_fd(tail, head)
            hoppings.append(self._get_edge(tail, head))

        #### Assemble and return result.
        result = InfiniteSystem()
        result.cell_size = cell_size
        result.sites = sites
        result.graph = g
        result.hoppings = hoppings
        result.onsite_hamiltonians = onsite_hamiltonians
        result.symmetry = self.symmetry
        return result


################ Finalized systems

class FiniteSystem(system.FiniteSystem):
    """
    Finalized `Builder` with leads.

    Usable as input for the solvers in `kwant.solvers`.
    """
    def hamiltonian(self, i, j, *args):
        if i == j:
            value = self.onsite_hamiltonians[i]
            if callable(value):
                value = value(self.sites[i], *args)
        else:
            edge_id = self.graph.first_edge_id(i, j)
            value = self.hoppings[edge_id]
            conj = value is Other
            if conj:
                i, j = j, i
                edge_id = self.graph.first_edge_id(i, j)
                value = self.hoppings[edge_id]
            if callable(value):
                sites = self.sites
                value = value(sites[i], sites[j], *args)
            if conj:
                value = herm_conj(value)
        return value

    def site(self, i):
        return self.sites[i]

    def pos(self, i):
        return self.sites[i].pos


class InfiniteSystem(system.InfiniteSystem):
    """Finalized infinite system, extracted from a `Builder`."""
    def hamiltonian(self, i, j, *args):
        if i == j:
            if i >= self.cell_size:
                i -= self.cell_size
            value = self.onsite_hamiltonians[i]
            if callable(value):
                value = value(self.symmetry.to_fd(self.sites[i]), *args)
        else:
            edge_id = self.graph.first_edge_id(i, j)
            value = self.hoppings[edge_id]
            conj = value is Other
            if conj:
                i, j = j, i
                edge_id = self.graph.first_edge_id(i, j)
                value = self.hoppings[edge_id]
            if callable(value):
                sites = self.sites
                site_i = sites[i]
                site_j = sites[j]
                site_i, site_j = self.symmetry.to_fd(site_i, site_j)
                value = value(site_i, site_j, *args)
            if conj:
                value = herm_conj(value)
        return value

    def site(self, i):
        return self.sites[i]

    def pos(self, i):
        return self.sites[i].pos



# Tutorial 2.7.2. 3D example: zincblende structure
# ================================================
#
# Physical background
# -------------------
#  - 3D Bravais lattices
#
# Kwant features highlighted
# --------------------------
#  - demonstrate different ways of plotting in 3D

import kwant
from matplotlib import pyplot

#HIDDEN_BEGIN_zincblende1
lat = kwant.lattice.general([(0, 0.5, 0.5), (0.5, 0, 0.5), (0.5, 0.5, 0)],
                            [(0, 0, 0), (0.25, 0.25, 0.25)])
a, b = lat.sublattices
#HIDDEN_END_zincblende1

#HIDDEN_BEGIN_zincblende2
def make_cuboid(a=15, b=10, c=5):
    def cuboid_shape(pos):
        x, y, z = pos
        return 0 <= x < a and 0 <= y < b and 0 <= z < c

    sys = kwant.Builder()
    sys[lat.shape(cuboid_shape, (0, 0, 0))] = None
    sys[lat.neighbors()] = None

    return sys
#HIDDEN_END_zincblende2


def main():
    # the standard plotting style for 3D is mainly useful for
    # checking shapes:
#HIDDEN_BEGIN_plot1
    sys = make_cuboid()

    kwant.plot(sys)
#HIDDEN_END_plot1

    # visualize the crystal structure better for a very small system
#HIDDEN_BEGIN_plot2
    sys = make_cuboid(a=1.5, b=1.5, c=1.5)

    def family_colors(site):
        return 'r' if site.family == a else 'g'

    kwant.plot(sys, site_size=0.18, site_lw=0.01, hop_lw=0.05,
               site_color=family_colors)
#HIDDEN_END_plot2


# Call the main function if the script gets executed (as opposed to imported).
# See <http://docs.python.org/library/__main__.html>.
if __name__ == '__main__':
    main()



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

cdef extern from "mumps_c_types.h":
     ctypedef int MUMPS_INT

     ctypedef float SMUMPS_REAL
     ctypedef float SMUMPS_COMPLEX

     ctypedef double DMUMPS_REAL
     ctypedef double DMUMPS_COMPLEX

     ctypedef struct mumps_complex:
         pass

     ctypedef struct mumps_double_complex:
         pass

     ctypedef float CMUMPS_REAL
     ctypedef mumps_complex CMUMPS_COMPLEX

     ctypedef double ZMUMPS_REAL
     ctypedef mumps_double_complex ZMUMPS_COMPLEX


cdef extern from "zmumps_c.h":
     ctypedef struct ZMUMPS_STRUC_C:
         MUMPS_INT sym, par, job
         MUMPS_INT comm_fortran
         MUMPS_INT icntl[40]
         ZMUMPS_REAL cntl[15]

         MUMPS_INT n

         MUMPS_INT nz
         MUMPS_INT *irn, *jcn
         ZMUMPS_COMPLEX *a

         MUMPS_INT nrhs, lrhs
         ZMUMPS_COMPLEX *rhs

         MUMPS_INT info[40], infog[40]
         ZMUMPS_REAL rinfo[40], rinfog[40]

         MUMPS_INT nz_rhs
         ZMUMPS_COMPLEX *rhs_sparse
         MUMPS_INT *irhs_sparse, *irhs_ptr

         MUMPS_INT size_schur, *listvar_schur
         ZMUMPS_COMPLEX *schur

     cdef void zmumps_c(ZMUMPS_STRUC_C *)



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

cdef extern from "defs.h":
    ctypedef signed int gint



# Tutorial 2.4.1. Band structure calculations
# ===========================================
#
# Physics background
# ------------------
#  band structure of a simple quantum wire in tight-binding approximation
#
# Kwant features highlighted
# --------------------------
#  - Computing the band structure of a finalized lead.

import kwant

# For plotting.
from matplotlib import pyplot

#HIDDEN_BEGIN_zxip
def make_lead(a=1, t=1.0, W=10):
    # Start with an empty lead with a single square lattice
    lat = kwant.lattice.square(a)

    sym_lead = kwant.TranslationalSymmetry((-a, 0))
    lead = kwant.Builder(sym_lead)

    # build up one unit cell of the lead, and add the hoppings
    # to the next unit cell
    for j in xrange(W):
        lead[lat(0, j)] = 4 * t

        if j > 0:
            lead[lat(0, j), lat(0, j - 1)] = -t

        lead[lat(1, j), lat(0, j)] = -t

    return lead
#HIDDEN_END_zxip


#HIDDEN_BEGIN_pejz
def main():
    lead = make_lead().finalized()
    kwant.plotter.bands(lead, show=False)
    pyplot.xlabel("momentum [(lattice constant)^-1]")
    pyplot.ylabel("energy [t]")
    pyplot.show()
#HIDDEN_END_pejz


# Call the main function if the script gets executed (as opposed to imported).
# See <http://docs.python.org/library/__main__.html>.
if __name__ == '__main__':
    main()



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

from nose.tools import assert_equal, assert_true
import numpy as np
# from kwant.graph import Graph, dissection
# from kwant.graph.dissection import edge_dissection

def _DISABLED_test_edge_dissection():
    # REMARK: This test is somewhat limited in the sense that it can only test
    #         for the general sanity of the output, not if the disection is
    #         really good (balanced, etc.).  Sanity is checked by making sure
    #         that every node is included exactly once in the tree.
    size = 5
    graph = Graph()

    for i in xrange(size - 1):
        offset = i * size
        for j in xrange(size - 1):
            graph.add_edge(offset + j, offset + j + 1)
            graph.add_edge(offset + j + 1, offset + j)
        if i > 0:
            for j in xrange(size):
                graph.add_edge(offset + j, offset + j - size)
                graph.add_edge(offset + j - size, offset + j)
    g = graph.compressed()

    tree = edge_dissection(g, 1)
    found = np.zeros(g.num_nodes, dtype = int)

    def parse_tree(entry):
        if type(entry) is tuple:
            parse_tree(entry[0])
            parse_tree(entry[1])
        else:
            found[entry] += 1

    parse_tree(tree)
    assert_true((found == 1).all())



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

from defs cimport gint

cdef extern from "c_slicer/slicer.h":
   struct Slicing:
      int nslices
      int *slice_ptr, *slices

   Slicing *slice(gint, gint *, gint *, gint, gint *,
                  gint, gint *)

   void freeSlicing(Slicing *)



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

ctypedef int l_int
ctypedef int l_logical

cdef extern:
  void sgetrf_(l_int *, l_int *, float *, l_int *, l_int *, l_int *)
  void dgetrf_(l_int *, l_int *, double *, l_int *, l_int *, l_int *)
  void cgetrf_(l_int *, l_int *, float complex *, l_int *, l_int *,
               l_int *)
  void zgetrf_(l_int *, l_int *, double complex *, l_int *, l_int *,
               l_int *)

  void sgetrs_(char *, l_int *, l_int *, float *, l_int *, l_int *,
               float *, l_int *, l_int *)
  void dgetrs_(char *, l_int *, l_int *, double *, l_int *, l_int *,
               double *, l_int *, l_int *)
  void cgetrs_(char *, l_int *, l_int *, float complex *, l_int *,
               l_int *, float complex *, l_int *, l_int *)
  void zgetrs_(char *, l_int *, l_int *, double complex *, l_int *,
               l_int *, double complex *, l_int *, l_int *)

  void sgecon_(char *, l_int *, float *, l_int *, float *, float *,
               float *, l_int *, l_int *)
  void dgecon_(char *, l_int *, double *, l_int *, double *, double *,
               double *, l_int *, l_int *)
  void cgecon_(char *, l_int *, float complex *, l_int *, float *,
               float *, float complex *, float *, l_int *)
  void zgecon_(char *, l_int *, double complex *, l_int *, double *,
               double *, double complex *, double *, l_int *)

  void sggev_(char *, char *, l_int *, float *, l_int *, float *, l_int *,
              float *, float *, float *, float *, l_int *, float *, l_int *,
              float *, l_int *, l_int *)
  void dggev_(char *, char *, l_int *, double *, l_int *, double *, l_int *,
              double *, double *, double *, double *, l_int *,
              double *, l_int *, double *, l_int *, l_int *)
  void cggev_(char *, char *, l_int *, float complex *, l_int *,
              float complex *, l_int *, float complex *, float complex *,
              float complex *, l_int *, float complex *, l_int *,
              float complex *, l_int *, float *, l_int *)
  void zggev_(char *, char *, l_int *, double complex *, l_int *,
              double complex *, l_int *, double complex *,
              double complex *, double complex *, l_int *,
              double complex *, l_int *, double complex *, l_int *,
              double *, l_int *)

  void sgees_(char *, char *, l_logical (*)(float *, float *),
              l_int *, float *, l_int *, l_int *,
              float *, float *, float *, l_int *,
              float *, l_int *, l_logical *, l_int *)
  void dgees_(char *, char *, l_logical (*)(double *, double *),
              l_int *, double *, l_int *, l_int *,
              double *, double *, double *, l_int *,
              double *, l_int *, l_logical *, l_int *)
  void cgees_(char *, char *,
              l_logical (*)(float complex *),
              l_int *, float complex *,
              l_int *, l_int *, float complex *,
              float complex *, l_int *,
              float complex *, l_int *, float *,
              l_logical *, l_int *)
  void zgees_(char *, char *,
              l_logical (*)(double complex *),
              l_int *, double complex *,
              l_int *, l_int *, double complex *,
              double complex *, l_int *,
              double complex *, l_int *,
              double *, l_logical *, l_int *)

  void strsen_(char *, char *, l_logical *, l_int *,
               float *, l_int *, float *,
               l_int *, float *, float *, l_int *,
               float *, float *, float *, l_int *,
               l_int *, l_int *, l_int *)
  void dtrsen_(char *, char *, l_logical *,
               l_int *, double *, l_int *,
               double *, l_int *, double *, double *,
               l_int *, double *, double *, double *,
               l_int *, l_int *, l_int *, l_int *)
  void ctrsen_(char *, char *, l_logical *,
               l_int *, float complex *,
               l_int *, float complex *,
               l_int *, float complex *, l_int *,
               float *, float *, float complex *,
               l_int *, l_int *)
  void ztrsen_(char *, char *, l_logical *,
               l_int *, double complex *,
               l_int *, double complex *,
               l_int *, double complex *, l_int *,
               double *, double *, double complex *,
               l_int *, l_int *)

  void strevc_(char *, char *, l_logical *,
               l_int *, float *, l_int *,
               float *, l_int *, float *, l_int *,
               l_int *, l_int *, float *, l_int *)
  void dtrevc_(char *, char *, l_logical *,
               l_int *, double *, l_int *,
               double *, l_int *, double *,
               l_int *, l_int *, l_int *, double *,
               l_int *)
  void ctrevc_(char *, char *, l_logical *,
               l_int *, float complex *,
               l_int *, float complex *,
               l_int *, float complex *,
               l_int *, l_int *, l_int *,
               float complex *, float *, l_int *)
  void ztrevc_(char *, char *, l_logical *,
               l_int *, double complex *,
               l_int *, double complex *,
               l_int *, double complex *,
               l_int *, l_int *, l_int *,
               double complex *, double *, l_int *)

  void sgges_(char *, char *, char *,
              l_logical (*)(float *, float *, float *),
              l_int *, float *, l_int *, float *,
              l_int *, l_int *, float *, float *,
              float *, float *, l_int *, float *,
              l_int *, float *, l_int *, l_logical *,
              l_int *)
  void dgges_(char *, char *, char *,
              l_logical (*)(double *, double *, double *),
              l_int *, double *, l_int *, double *,
              l_int *, l_int *, double *, double *,
              double *, double *, l_int *, double *,
              l_int *, double *, l_int *,
              l_logical *, l_int *)
  void cgges_(char *, char *, char *,
              l_logical (*)(float complex *, float complex *),
              l_int *, float complex *,
              l_int *, float complex *,
              l_int *, l_int *, float complex *,
              float complex *, float complex *,
              l_int *, float complex *,
              l_int *, float complex *,
              l_int *, float *, l_logical *, l_int *)
  void zgges_(char *, char *, char *,
              l_logical (*)(double complex *, double complex *),
              l_int *, double complex *,
              l_int *, double complex *,
              l_int *, l_int *, double complex *,
              double complex *,
              double complex *, l_int *,
              double complex *, l_int *,
              double complex *, l_int *,
              double *, l_logical *, l_int *)

  void stgsen_(l_int *, l_logical *,
               l_logical *, l_logical *,
               l_int *, float *, l_int *, float *,
               l_int *, float *, float *, float *,
               float *, l_int *, float *, l_int *,
               l_int *, float *, float *, float *, float *,
               l_int *, l_int *, l_int *, l_int *)
  void dtgsen_(l_int *, l_logical *,
               l_logical *, l_logical *,
               l_int *, double *, l_int *,
               double *, l_int *, double *, double *,
               double *, double *, l_int *, double *,
               l_int *, l_int *, double *, double *,
               double *, double *, l_int *, l_int *,
               l_int *, l_int *)
  void ctgsen_(l_int *, l_logical *,
               l_logical *, l_logical *,
               l_int *, float complex *,
               l_int *, float complex *,
               l_int *, float complex *,
               float complex *,
               float complex *, l_int *,
               float complex *, l_int *, l_int *,
               float *, float *, float *,
               float complex *, l_int *, l_int *,
               l_int *, l_int *)
  void ztgsen_(l_int *, l_logical *,
               l_logical *, l_logical *,
               l_int *, double complex *,
               l_int *, double complex *,
               l_int *, double complex *,
               double complex *,
               double complex *, l_int *,
               double complex *, l_int *, l_int *,
               double *, double *, double *,
               double complex *, l_int *, l_int *,
               l_int *, l_int *)

  void stgevc_(char *, char *, l_logical *,
               l_int *, float *, l_int *,
               float *, l_int *, float *,
               l_int *, float *, l_int *,
               l_int *, l_int *, float *, l_int *)
  void dtgevc_(char *, char *, l_logical *,
               l_int *, double *, l_int *,
               double *, l_int *, double *,
               l_int *, double *, l_int *,
               l_int *, l_int *, double *, l_int *)
  void ctgevc_(char *, char *, l_logical *,
               l_int *, float complex *,
               l_int *, float complex *,
               l_int *, float complex *,
               l_int *, float complex *,
               l_int *, l_int *, l_int *,
               float complex *, float *, l_int *)
  void ztgevc_(char *, char *, l_logical *,
               l_int *, double complex *,
               l_int *, double complex *,
               l_int *, double complex *,
               l_int *, double complex *,
               l_int *, l_int *, l_int *,
               double complex *, double *, l_int *)



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

import numpy as np
cimport numpy as np
cimport cython
from libc.string cimport memcpy
from kwant.graph.defs cimport gint
from .defs import gint_dtype
from kwant.graph.core cimport CGraph
cimport kwant.graph.c_slicer as c_slicer

__all__ = ['slice']

@cython.boundscheck(False)
def slice(CGraph graph, left, right):
    """
    TODO: write me.
    """
    cdef np.ndarray[gint, ndim=1] leftarr, rightarr, slc
    cdef c_slicer.Slicing *slicing
    cdef int i, slc_size

    leftarr = np.unique(np.array(left, dtype=gint_dtype))
    rightarr = np.unique(np.array(right, dtype=gint_dtype))

    if leftarr.ndim != 1:
        raise ValueError("Left cannot be interpreted as a 1D array.")

    if rightarr.ndim != 1:
        raise ValueError("Right cannot be interpreted as a 1D array.")

    if leftarr.size == 0 or rightarr.size == 0:
        raise ValueError("Empty boundary arrays are not supported yet.")

    # slicing only possible if there is no overlap between
    # left and right slices
    if np.intersect1d(rightarr, leftarr, assume_unique=True).size:
        return [tuple(xrange(graph.num_nodes))]

    slicing = c_slicer.slice(graph.num_nodes,
                             graph.heads_idxs,
                             graph.heads,
                             leftarr.size, <gint *>leftarr.data,
                             rightarr.size, <gint *>rightarr.data)
    slices = []
    for i in xrange(slicing.nslices):
        slc_size = slicing.slice_ptr[i+1] - slicing.slice_ptr[i]
        slc = np.empty(slc_size, dtype=gint_dtype)
        memcpy(slc.data, slicing.slices + slicing.slice_ptr[i],
               sizeof(gint) * slc_size)
        slices.append(slc)
    c_slicer.freeSlicing(slicing)
    return slices



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

from __future__ import division
from math import sin, cos, sqrt, pi, copysign
from collections import namedtuple
from itertools import izip
import numpy as np
import numpy.linalg as npl
import scipy.linalg as la
from .. import linalg as kla

dot = np.dot

__all__ = ['selfenergy', 'modes', 'PropagatingModes', 'StabilizedModes']


# Container classes
Linsys = namedtuple('Linsys', ['eigenproblem', 'v', 'extract'])


class PropagatingModes(object):
    """The calculated propagating modes of a lead.

    Attributes
    ----------
    wave_functions : numpy array
        The wave functions of the propagating modes.
    momenta : numpy array
        Momenta of the modes.
    velocities : numpy array
        Velocities of the modes.

    Notes
    =====
    The sort order of all the three arrays is identical. The first half of the
    modes have negative velocity, the second half have positive velocity. The
    modes with negative velocity are ordered from larger to lower momenta, the
    modes with positive velocity vice versa.

    The first dimension of `wave_functions` corresponds to the orbitals of all
    the sites in a unit cell, the second one to the number of the mode.  Each
    mode is normalized to carry unit current. If several modes have the same
    momentum and velocity, an arbitrary orthonormal basis in the subspace of
    these modes is chosen.
    """
    def __init__(self, wave_functions, velocities, momenta):
        kwargs = locals()
        kwargs.pop('self')
        self.__dict__.update(kwargs)


class StabilizedModes(object):
    """Stabilized eigendecomposition of the translation operator.

    Due to the lack of Hermiticity of the translation operator, its
    eigendecomposition is frequently poorly conditioned. Solvers in Kwant use
    this stabilized decomposition of the propagating and evanescent modes in
    the leads. If the hopping between the unit cells of an infinite system is
    invertible, the translation eigenproblem is written in the basis `psi_n,
    h_hop^+ * psi_(n+1)`, with ``h_hop`` the hopping between unit cells.  If
    `h_hop` is not invertible, and has the singular value decomposition `u s
    v^+`, then the eigenproblem is written in the basis `sqrt(s) v^+ psi_n,
    sqrt(s) u^+ psi_(n+1)`. In this basis we calculate the eigenvectors of the
    propagating modes, and the Schur vectors (an orthogonal basis) in the space
    of evanescent modes.

    `vecs` and `vecslmbdainv` are the first and the second halves of the wave
    functions.  The first `nmodes` are eigenmodes moving in the negative
    direction (hence they are incoming into the system in Kwant convention),
    the second `nmodes` are eigenmodes moving in the positive direction. The
    remaining modes are the Schur vectors of the modes evanescent in the
    positive direction. Propagating modes with the same eigenvalue are
    orthogonalized, and all the propagating modes are normalized to carry unit
    current. Finally the `sqrt_hop` attribute is `v sqrt(s)`.

    Attributes
    ----------
    vecs : numpy array
        Translation eigenvectors.
    vecslmbdainv : numpy array
        Translation eigenvectors divided by the corresponding eigenvalues.
    nmodes : int
        Number of left-moving (or right-moving) modes.
    sqrt_hop : numpy array or None
        Part of the SVD of `h_hop`, or None if the latter is invertible.
    """

    def __init__(self, vecs, vecslmbdainv, nmodes, sqrt_hop=None):
        kwargs = locals()
        kwargs.pop('self')
        self.__dict__.update(kwargs)

    def selfenergy(self):
        """
        Compute the self-energy generated by lead modes.

        Returns
        -------
        Sigma : numpy array, real or complex, shape (M,M)
            The computed self-energy. Note that even if `h_cell` and `h_hop` are
            both real, `Sigma` will typically be complex. (More precisely, if
            there is a propagating mode, `Sigma` will definitely be complex.)
        """
        v = self.sqrt_hop
        vecs = self.vecs[:, self.nmodes:]
        vecslmbdainv = self.vecslmbdainv[:, self.nmodes:]
        if v is not None:
            return dot(v, dot(vecs, la.solve(vecslmbdainv, v.T.conj())))
        else:
            return la.solve(vecslmbdainv.T, vecs.T).T


# Auxiliary functions that perform different parts of the calculation.
def setup_linsys(h_cell, h_hop, tol=1e6, stabilization=None):
    """Make an eigenvalue problem for eigenvectors of translation operator.

    Parameters
    ----------
    h_cell : numpy array with shape (n, n)
        Hamiltonian of a single lead unit cell.
    h_hop : numpy array with shape (n, m), m <= n
        Hopping Hamiltonian from a cell to the next one.
    tol : float
        Numbers are considered zero when they are smaller than `tol` times
        the machine precision.
    stabilization : list of 2 booleans or None
        Which steps of the eigenvalue problem stabilization to perform. If the
        value is `None`, then Kwant chooses the fastest (and least stable)
        algorithm that is expected to be sufficient.  For any other value,
        Kwant forms the eigenvalue problem in the basis of the hopping singular
        values.  The first element set to `True` forces Kwant to add an
        anti-Hermitian term to the cell Hamiltonian before inverting. If it is
        set to `False`, the extra term will only be added if the cell
        Hamiltonian isn't invertible. The second element set to `True` forces
        Kwant to solve a generalized eigenvalue problem, and not to reduce it
        to the regular one.  If it is `False`, reduction to a regular problem
        is performed if possible.

    Returns
    -------
    linsys : namedtuple
        A named tuple containing `matrices` a matrix pencil defining
        the eigenproblem, `v` a hermitian conjugate of the last matrix in
        the hopping singular value decomposition, and functions for
        extracting the wave function in the unit cell from the wave function
        in the basis of the nonzero singular exponents of the hopping.

    Notes
    -----
    The lead problem with degenerate hopping is rather complicated, and the
    details of the algorithm will be published elsewhere.

    """
    n = h_cell.shape[0]
    m = h_hop.shape[1]
    if stabilization is not None:
        stabilization = list(stabilization)

    if not (np.any(h_hop.real) or np.any(h_hop.imag)):
        # Inter-cell hopping is zero.  The current algorithm is not suited to
        # treat this extremely singular case.
        # Note: np.any(h_hop) returns (at least from numpy 1.6.*)
        #       False if h_hop is purely imaginary
        raise ValueError("Inter-cell hopping is exactly zero.")

    # If both h and t are real, it may be possible to use the real eigenproblem.
    if (not np.any(h_hop.imag)) and (not np.any(h_cell.imag)):
        h_hop = h_hop.real
        h_cell = h_cell.real

    eps = np.finfo(np.common_type(h_cell, h_hop)).eps * tol

    # First check if the hopping matrix has singular values close to 0.
    # (Close to zero is defined here as |x| < eps * tol * s[0] , where
    # s[0] is the largest singular value.)

    u, s, vh = la.svd(h_hop)
    assert m == vh.shape[1], "Corrupt output of svd."
    n_nonsing = np.sum(s > eps * s[0])

    if (n_nonsing == n and stabilization is None):
        # The hopping matrix is well-conditioned and can be safely inverted.
        # Hence the regular transfer matrix may be used.
        hop_inv = la.inv(h_hop)

        A = np.zeros((2*n, 2*n), dtype=np.common_type(h_cell, h_hop))
        A[:n, :n] = dot(hop_inv, -h_cell)
        A[:n, n:] = -hop_inv
        A[n:, :n] = h_hop.T.conj()

        # The function that can extract the full wave function psi from the
        # projected one. Here it is almost trivial, but used for simplifying
        # the logic.

        def extract_wf(psi, lmbdainv):
            return np.copy(psi[:n])

        matrices = (A, None)
        v_out = None
    else:
        if stabilization is None:
            stabilization = [None, False]

        # The hopping matrix has eigenvalues close to 0 - those
        # need to be eliminated.

        # Recast the svd of h_hop = u s v^dagger such that
        # u, v are matrices with shape n x n_nonsing.
        u = u[:, :n_nonsing]
        s = s[:n_nonsing]
        u = u * np.sqrt(s)
        # pad v with zeros if necessary
        v = np.zeros((n, n_nonsing), dtype=vh.dtype)
        v[:vh.shape[1]] = vh[:n_nonsing].T.conj()
        v = v * np.sqrt(s)

        # Eliminating the zero eigenvalues requires inverting the on-site
        # Hamiltonian, possibly including a self-energy-like term.  The
        # self-energy-like term stabilizes the inversion, but the most stable
        # choice is inherently complex. This can be disadvantageous if the
        # Hamiltonian is real, since staying in real arithmetics can be
        # significantly faster.  The strategy here is to add a complex
        # self-energy-like term always if the original Hamiltonian is complex,
        # and check for invertibility first if it is real

        matrices_real = issubclass(np.common_type(h_cell, h_hop), np.floating)
        add_imaginary = stabilization[0] or ((stabilization[0] is None) and
                                             not matrices_real)
        # Check if there is a chance we will not need to add an imaginary term.
        if not add_imaginary:
            h = h_cell
            sol = kla.lu_factor(h)
            rcond = kla.rcond_from_lu(sol, npl.norm(h, 1))

            if rcond < eps:
                need_to_stabilize = True
            else:
                need_to_stabilize = False

        if add_imaginary or need_to_stabilize:
            need_to_stabilize = True
            # Matrices are complex or need self-energy-like term to be
            # stabilized.
            temp = dot(u, u.T.conj()) + dot(v, v.T.conj())
            h = h_cell + 1j * temp

            sol = kla.lu_factor(h)
            rcond = kla.rcond_from_lu(sol, npl.norm(h, 1))

            # If the condition number of the stabilized h is
            # still bad, there is nothing we can do.
            if rcond < eps:
                raise RuntimeError("Flat band encountered at the requested "
                                   "energy, result is badly defined.")

        # The function that can extract the full wave function psi from
        # the projected one (v^dagger psi lambda^-1, s u^dagger psi).

        def extract_wf(psi, lmbdainv):
            wf = - dot(u, psi[: n_nonsing] * lmbdainv) - \
                 dot(v, psi[n_nonsing:])
            if need_to_stabilize:
                wf += 1j * (dot(v, psi[: n_nonsing]) +
                            dot(u, psi[n_nonsing:] * lmbdainv))
            return kla.lu_solve(sol, wf)

        # Setup the generalized eigenvalue problem.

        A = np.zeros((2 * n_nonsing, 2 * n_nonsing), np.common_type(h, h_hop))
        B = np.zeros((2 * n_nonsing, 2 * n_nonsing), np.common_type(h, h_hop))

        begin, end = slice(n_nonsing), slice(n_nonsing, None)

        A[end, begin] = np.identity(n_nonsing)
        temp = kla.lu_solve(sol, v)
        temp2 = dot(u.T.conj(), temp)
        if need_to_stabilize:
            A[begin, begin] = -1j * temp2
        A[begin, end] = temp2
        temp2 = dot(v.T.conj(), temp)
        if need_to_stabilize:
            A[end, begin] -= 1j *temp2
        A[end, end] = temp2

        B[begin, end] = -np.identity(n_nonsing)
        temp = kla.lu_solve(sol, u)
        temp2 = dot(u.T.conj(), temp)
        B[begin, begin] = -temp2
        if need_to_stabilize:
            B[begin, end] += 1j * temp2
        temp2 = dot(v.T.conj(), temp)
        B[end, begin] = -temp2
        if need_to_stabilize:
            B[end, end] = 1j * temp2

        v_out = v[:m]

        # Solving a generalized eigenproblem is about twice as expensive
        # as solving a regular eigenvalue problem.
        # Computing the LU factorization is negligible compared to both
        # (approximately 1/30th of a regular eigenvalue problem).
        # Because of this, it makes sense to try to reduce
        # the generalized eigenvalue problem to a regular one, provided
        # the matrix B can be safely inverted.

        lu_b = kla.lu_factor(B)
        if not stabilization[1]:
            rcond = kla.rcond_from_lu(lu_b, npl.norm(B, 1))
            # A more stringent condition is used here since errors can
            # accumulate from here to the eigenvalue calculation later.
            stabilization[1] = rcond > eps * tol

        if stabilization[1]:
            matrices = (kla.lu_solve(lu_b, A), None)
        else:
            matrices = (A, B)
    return Linsys(matrices, v_out, extract_wf)


def unified_eigenproblem(a, b=None, tol=1e6):
    """A helper routine for modes(), that wraps eigenproblems.

    This routine wraps the regular and general eigenproblems that can arise
    in a unfied way.

    Parameters
    ----------
    a : numpy array
        The matrix on the left hand side of a regular or generalized eigenvalue
        problem.
    b : numpy array or None
        The matrix on the right hand side of the generalized eigenvalue problem.
    tol : float
        The tolerance for separating eigenvalues with absolute value 1 from the
        rest.

    Returns
    -------
    ev : numpy array
        An array of eigenvalues (can contain NaNs and Infs, but those
        are not accessed in `modes()`) The number of eigenvalues equals
        twice the number of nonzero singular values of
        `h_hop` (so `2*h_cell.shape[0]` if `h_hop` is invertible).
    evanselect : numpy integer array
        Index array of right-decaying modes.
    propselect : numpy integer array
        Index array of propagating modes (both left and right).
    vec_gen(select) : function
        A function that computes the eigenvectors chosen by the array select.
    ord_schur(select) : function
        A function that computes the unitary matrix (corresponding to the right
        eigenvector space) of the (general) Schur decomposition reordered such
        that the eigenvalues chosen by the array select are in the top left
        block.
    """
    if b is None:
        eps = np.finfo(a.dtype).eps * tol
        t, z, ev = kla.schur(a)

        # Right-decaying modes.
        select = abs(ev) > 1 + eps
        # Propagating modes.
        propselect = abs(abs(ev) - 1) < eps

        vec_gen = lambda x: kla.evecs_from_schur(t, z, select=x)
        ord_schur = lambda x: kla.order_schur(x, t, z, calc_ev=False)[1]

    else:
        eps = np.finfo(np.common_type(a, b)).eps * tol
        s, t, z, alpha, beta = kla.gen_schur(a, b, calc_q=False)

        # Right-decaying modes.
        select = abs(alpha) > (1 + eps) * abs(beta)
        # Propagating modes.
        propselect = (abs(abs(alpha) - abs(beta)) < eps * abs(beta))

        warning_settings = np.seterr(divide='ignore', invalid='ignore')
        ev = alpha / beta
        np.seterr(**warning_settings)
        # Note: the division is OK here, since we later only access
        #       eigenvalues close to the unit circle

        vec_gen = lambda x: kla.evecs_from_gen_schur(s, t, z=z, select=x)
        ord_schur = lambda x: kla.order_gen_schur(x, s, t, z=z,
                                                  calc_ev=False)[2]

    return ev, select, propselect, vec_gen, ord_schur


def make_proper_modes(lmbdainv, psi, extract, tol=1e6):
    """
    Find, normalize and sort the propagating eigenmodes.

    Special care is taken of the case of degenerate k-values, where the
    numerically computed modes are typically a superposition of the real
    modes. In this case, also the proper (orthogonal) modes are computed.
    """
    vel_eps = np.finfo(psi.dtype).eps * tol

    nmodes = psi.shape[1]
    n = len(psi) // 2

    # Array for the velocities.
    velocities = np.empty(nmodes, dtype=float)

    # Calculate the full wave function in real space.
    full_psi = extract(psi, lmbdainv)

    # Find clusters of nearby eigenvalues. Since the eigenvalues occupy the
    # unit circle, special care has to be taken to not introduce a cut at
    # lambda = -1.
    eps = np.finfo(lmbdainv.dtype).eps * tol
    angles = np.angle(lmbdainv)
    sort_order = np.resize(np.argsort(angles), (2 * len(angles,)))
    boundaries = np.argwhere(np.abs(np.diff(lmbdainv[sort_order]))
                             > eps).flatten() + 1

    # Detect the singular case of all eigenvalues equal.
    if boundaries.shape == (0,) and len(angles):
        boundaries = np.array([0, len(angles)])

    for interval in izip(boundaries[:-1], boundaries[1:]):
        if interval[1] > boundaries[0] + len(angles):
            break

        indx = sort_order[interval[0] : interval[1]]

        # If there is a degenerate eigenvalue with several different
        # eigenvectors, the numerical routines return some arbitrary
        # overlap of the real, physical solutions. In order
        # to figure out the correct wave function, we need to
        # have the full, not the projected wave functions
        # (at least to our current knowledge).

        # Finding the true modes is done in two steps:

        # 1. The true transversal modes should be orthogonal to each other, as
        # they share the same Bloch momentum (note that transversal modes with
        # different Bloch momenta k1 and k2 need not be orthogonal, the full
        # modes are orthogonal because of the longitudinal dependence e^{i k1
        # x} and e^{i k2 x}).  The modes with the same k are therefore
        # orthogonalized. Moreover for the velocity to have a proper value the
        # modes should also be normalized.

        q, r = la.qr(full_psi[:, indx], mode='economic')

        # If the eigenvectors were purely real up to this stage,
        # they will typically become complex after the rotation.
        if psi.dtype != np.common_type(psi, r):
            psi = psi.astype(np.common_type(psi, r))
        if full_psi.dtype != np.common_type(full_psi, q):
            full_psi = full_psi.astype(np.common_type(psi, q))

        full_psi[:, indx] = q
        psi[:, indx] = la.solve(r.T, psi[:, indx].T).T

        # 2. Moving infinitesimally away from the degeneracy
        # point, the modes should diagonalize the velocity
        # operator (i.e. when they are non-degenerate any more)
        # The modes are therefore rotated properly such that they
        # diagonalize the velocity operator.
        # Note that step 2. does not give a unique result if there are
        # two modes with the same velocity, or if the modes stay
        # degenerate even for a range of Bloch momenta (and hence
        # must have the same velocity). However, this does not matter,
        # since we are happy with any superposition in this case.

        vel_op = -1j * dot(psi[n:, indx].T.conj(), psi[:n, indx])
        vel_op = vel_op + vel_op.T.conj()
        vel_vals, rot = la.eigh(vel_op)

        # If the eigenvectors were purely real up to this stage,
        # they will typically become complex after the rotation.

        if psi.dtype != np.common_type(psi, rot):
            psi = psi.astype(np.common_type(psi, rot))
        if full_psi.dtype != np.common_type(full_psi, rot):
            full_psi = full_psi.astype(np.common_type(psi, rot))

        psi[:, indx] = dot(psi[:, indx], rot)
        full_psi[:, indx] = dot(full_psi[:, indx], rot)
        velocities[indx] = vel_vals

    if np.any(abs(velocities) < vel_eps):
        raise RuntimeError("Found a mode with zero or close to zero velocity.")
    if 2 * np.sum(velocities < 0) != len(velocities):
        raise RuntimeError("Numbers of left- and right-propagating "
                           "modes differ, possibly due to a numerical "
                           "instability.")
    momenta = -np.angle(lmbdainv)
    order = np.lexsort([velocities, -np.sign(velocities) * momenta,
                        np.sign(velocities)])

    # The following is necessary due to wrong numpy handling of zero length
    # arrays, which is going to be fixed in numpy 1.8.
    if not len(order):
        order = slice(None)
    velocities = velocities[order]
    norm = np.sqrt(abs(velocities))
    full_psi = full_psi[:, order] / norm
    psi = psi[:, order] / norm
    momenta = momenta[order]

    return psi, PropagatingModes(full_psi, velocities, momenta)


def modes(h_cell, h_hop, tol=1e6, stabilization=None):
    """Compute the eigendecomposition of a translation operator of a lead.

    Parameters
    ----------
    h_cell : numpy array, real or complex, shape (N,N) The unit cell
        Hamiltonian of the lead unit cell.
    h_hop : numpy array, real or complex, shape (N,M)
        The hopping matrix from a lead cell to the one on which self-energy
        has to be calculated (and any other hopping in the same direction).
    tol : float
        Numbers and differences are considered zero when they are smaller
        than `tol` times the machine precision.
    stabilization : list of 2 booleans or None
        Which steps of the eigenvalue prolem stabilization to perform. If the
        value is `None`, then Kwant chooses the fastest (and least stable)
        algorithm that is expected to be sufficient.  For any other value,
        Kwant forms the eigenvalue problem in the basis of the hopping singular
        values.  The first element set to `True` forces Kwant to add an
        anti-Hermitian term to the cell Hamiltonian before inverting. If it is
        set to `False`, the extra term will only be added if the cell
        Hamiltonian isn't invertible. The second element set to `True` forces
        Kwant to solve a generalized eigenvalue problem, and not to reduce it
        to the regular one.  If it is `False`, reduction to a regular problem
        is performed if possible.  Selecting the stabilization manually is
        mostly necessary for testing purposes.

    Returns
    -------
    propagating : `~kwant.physics.PropagatingModes`
        Contains the array of the wave functions of propagating modes, their
        momenta, and their velocities. It can be used to identify the gauge in
        which the scattering problem is solved.
    stabilized : `~kwant.physics.StabilizedModes`
        A basis of propagating and evanescent modes used by the solvers.

    Notes
    -----
    The propagating modes are sorted according to the longitudinal component of
    their k-vector, with incoming modes having k sorted in descending order,
    and outgoing modes having k sorted in ascending order.  In simple cases
    where bands do not cross, this ordering corresponds to "lowest modes
    first". In general, however, it is necessary to examine the band structure
    -- something this function is not doing by design.

    Propagating modes with the same momentum are orthogonalized. All the
    propagating modes are normalized by current.

    This function uses the most stable and efficient algorithm for calculating
    the mode decomposition that the Kwant authors are aware about. Its details
    are to be published.
    """
    m = h_hop.shape[1]
    n = h_cell.shape[0]

    if (h_cell.shape[0] != h_cell.shape[1] or
        h_cell.shape[0] != h_hop.shape[0]):
        raise ValueError("Incompatible matrix sizes for h_cell and h_hop.")

    # Note: np.any(h_hop) returns (at least from numpy 1.6.1 - 1.8-devel)
    #       False if h_hop is purely imaginary
    if not (np.any(h_hop.real) or np.any(h_hop.imag)):
        v = np.empty((0, m))
        return PropagatingModes(np.empty((0, n)), np.empty((0,)),
                                np.empty((0,))), \
               StabilizedModes(np.empty((0, 0)), np.empty((0, 0)), 0, v)

    # Defer most of the calculation to helper routines.
    matrices, v, extract = setup_linsys(h_cell, h_hop, tol, stabilization)
    ev, evanselect, propselect, vec_gen, ord_schur =\
         unified_eigenproblem(*(matrices + (tol,)))

    if v is not None:
        n = v.shape[1]

    nrightmovers = np.sum(propselect) // 2
    nevan = n - nrightmovers
    evan_vecs = ord_schur(evanselect)[:, :nevan]

    # Compute the propagating eigenvectors.
    prop_vecs = vec_gen(propselect)
    # Compute their velocity, and, if necessary, rotate them
    prop_vecs, real_space_data = \
            make_proper_modes(ev[propselect], prop_vecs, extract, tol)

    vecs = np.c_[prop_vecs[n:], evan_vecs[n:]]
    vecslmbdainv = np.c_[prop_vecs[:n], evan_vecs[:n]]

    return real_space_data, StabilizedModes(vecs, vecslmbdainv, nrightmovers, v)


def selfenergy(h_cell, h_hop, tol=1e6):
    """
    Compute the self-energy generated by the lead.

    Parameters
    ----------
    h_cell : numpy array, real or complex, shape (N,N) The unit cell Hamiltonian
        of the lead unit cell.
    h_hop : numpy array, real or complex, shape (N,M)
        The hopping matrix from a lead cell to the one on which self-energy
        has to be calculated (and any other hopping in the same direction).
    tol : float
        Numbers are considered zero when they are smaller than `tol` times
        the machine precision.

    Returns
    -------
    Sigma : numpy array, real or complex, shape (M,M)
        The computed self-energy. Note that even if `h_cell` and `h_hop` are
        both real, `Sigma` will typically be complex. (More precisely, if there
        is a propagating mode, `Sigma` will definitely be complex.)

    Notes
    -----
    For simplicity this function internally calculates the modes first.
    This may cause a small slowdown, and can be improved if necessary.
    """
    propagating, stabilized = modes(h_cell, h_hop, tol)
    return stabilized.selfenergy()


def square_selfenergy(width, hopping, fermi_energy):
    """
    Calculate analytically the self energy for a square lattice.

    The lattice is assumed to have a single orbital per site and
    nearest-neighbor hopping.

    Parameters
    ----------
    width : integer
        width of the lattice
    """

    # Following appendix C of M. Wimmer's diploma thesis:
    # http://www.physik.uni-regensburg.de/forschung/\
    # richter/richter/media/research/publications2004/wimmer-Diplomarbeit.pdf

    # p labels transversal modes.  i and j label the sites of a cell.

    # Precalculate the transverse wave function.
    psi_p_i = np.empty((width, width))
    factor = pi / (width + 1)
    prefactor = sqrt(2 / (width + 1))
    for p in xrange(width):
        for i in xrange(width):
            psi_p_i[p, i] = prefactor * sin(factor * (p + 1) * (i + 1))

    # Precalculate the integrals of the longitudinal wave functions.
    def f(q):
        if abs(q) <= 2:
            return q/2 - 1j * sqrt(1 - (q / 2) ** 2)
        else:
            return q/2 - copysign(sqrt((q / 2) ** 2 - 1), q)
    f_p = np.empty((width,), dtype=complex)
    for p in xrange(width):
        e = 2 * hopping * (1 - cos(factor * (p + 1)))
        q = (fermi_energy - e) / hopping - 2
        f_p[p] = f(q)

    # Put everything together into the self energy and return it.
    result = np.empty((width, width), dtype=complex)
    for i in xrange(width):
        for j in xrange(width):
            result[i, j] = hopping * \
                (psi_p_i[:, i] * psi_p_i[:, j].conj() * f_p).sum()
    return result



# Tutorial 2.7.1. 2D example: graphene quantum dot
# ================================================
#
# Physics background
# ------------------
#  - graphene edge states
#
# Kwant features highlighted
# --------------------------
#  - demonstrate different ways of plotting

import kwant
from matplotlib import pyplot

#HIDDEN_BEGIN_makesys
lat = kwant.lattice.honeycomb()
a, b = lat.sublattices

def make_system(r=8, t=-1, tp=-0.1):

    def circle(pos):
        x, y = pos
        return x**2 + y**2 < r**2

    sys = kwant.Builder()
    sys[lat.shape(circle, (0, 0))] = 0
    sys[lat.neighbors()] = t
    sys.eradicate_dangling()
    if tp:
        sys[lat.neighbors(2)] = tp

    return sys
#HIDDEN_END_makesys


#HIDDEN_BEGIN_plotsys1
def plot_system(sys):
    kwant.plot(sys)
#HIDDEN_END_plotsys1
    # the standard plot is ok, but not very intelligible. One can do
    # better by playing wioth colors and linewidths

    # use color and linewidths to get a better plot
#HIDDEN_BEGIN_plotsys2
    def family_color(site):
        return 'black' if site.family == a else 'white'

    def hopping_lw(site1, site2):
        return 0.04 if site1.family == site2.family else 0.1

    kwant.plot(sys, site_lw=0.1, site_color=family_color, hop_lw=hopping_lw)
#HIDDEN_END_plotsys2


#HIDDEN_BEGIN_plotdata1
def plot_data(sys, n):
    import scipy.linalg as la

    sys = sys.finalized()
    ham = sys.hamiltonian_submatrix()
    evecs = la.eigh(ham)[1]

    wf = abs(evecs[:, n])**2
#HIDDEN_END_plotdata1

    # the usual - works great in general, looks just a bit crufty for
    # small systems

#HIDDEN_BEGIN_plotdata2
    kwant.plotter.map(sys, wf, oversampling=10, cmap='gist_heat_r')
#HIDDEN_END_plotdata2

    # use two different sort of triangles to cleanly fill the space
#HIDDEN_BEGIN_plotdata3
    def family_shape(i):
        site = sys.site(i)
        return ('p', 3, 180) if site.family == a else ('p', 3, 0)

    def family_color(i):
        return 'black' if sys.site(i).family == a else 'white'

    kwant.plot(sys, site_color=wf, site_symbol=family_shape,
               site_size=0.5, hop_lw=0, cmap='gist_heat_r')
#HIDDEN_END_plotdata3

    # plot by changing the symbols itself
#HIDDEN_BEGIN_plotdata4
    def site_size(i):
        return 3 * wf[i] / wf.max()

    kwant.plot(sys, site_size=site_size, site_color=(0, 0, 1, 0.3),
               hop_lw=0.1)
#HIDDEN_END_plotdata4


def main():
    # plot the graphene system in different styles
    sys = make_system()

    plot_system(sys)

    # compute a wavefunction (number 225) and plot it in different
    # styles
    sys = make_system(tp=0)

    plot_data(sys, 225)


# Call the main function if the script gets executed (as opposed to imported).
# See <http://docs.python.org/library/__main__.html>.
if __name__ == '__main__':
    main()



# Tutorial 2.6.1. Superconductors - "Orbital description": using matrices
# =======================================================================
#
# Physics background
# ------------------
#  band structure of a superconducting quantum wire in tight-binding
#  approximation
#
# Kwant features highlighted
# --------------------------
#  - Repetition of previously used concepts (band structure calculations,
#    matrices as values in Builder).
#  - Main motivation is to contrast to the implementation of superconductivity
#    in tutorial5b.py

import kwant

import numpy as np
import tinyarray

# For plotting
from matplotlib import pyplot

#HIDDEN_BEGIN_nbvn
tau_x = tinyarray.array([[0, 1], [1, 0]])
tau_z = tinyarray.array([[1, 0], [0, -1]])


def make_lead(a=1, t=1.0, mu=0.7, Delta=0.1, W=10):
    # Start with an empty lead with a single square lattice
    lat = kwant.lattice.square(a)

    sym_lead = kwant.TranslationalSymmetry((-a, 0))
    lead = kwant.Builder(sym_lead)

    # build up one unit cell of the lead, and add the hoppings
    # to the next unit cell
    for j in xrange(W):
        lead[lat(0, j)] = (4 * t - mu) * tau_z + Delta * tau_x

        if j > 0:
            lead[lat(0, j), lat(0, j - 1)] = -t * tau_z

        lead[lat(1, j), lat(0, j)] = -t * tau_z

    return lead
#HIDDEN_END_nbvn


def main():
    # Make system and finalize it right away.
    lead = make_lead().finalized()

    kwant.plotter.bands(lead, momenta=np.linspace(-1.5, 1.5, 101), show=False)
    pyplot.xlabel("momentum [(lattice constant)^-1]")
    pyplot.ylabel("energy [t]")
    pyplot.ylim([-0.8, 0.8])
    pyplot.show()


# Call the main function if the script gets executed (as opposed to imported).
# See <http://docs.python.org/library/__main__.html>.
if __name__ == '__main__':
    main()



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

import numpy as np
from nose.tools import assert_equal, assert_true
from kwant.graph import Graph
from kwant.graph.utils import \
 make_undirected, remove_duplicates, induced_subgraph
from kwant.graph.defs import gint_dtype

def test_make_undirected():
    graph = Graph(True)
    graph.add_edge(0, 1)
    graph.add_edge(1, 0)
    graph.add_edge(1, 2)
    graph.add_edge(2, -1)
    g = graph.compressed()

    # First, test with no duplicates removed,
    g2 = make_undirected(g, remove_dups=False)

    assert_equal(g2.num_nodes, g.num_nodes)
    assert_equal(g2.num_edges, 6)
    assert_true(g2.has_edge(0, 1))
    assert_true(g2.has_edge(1, 0))
    assert_true(g2.has_edge(1, 2))
    assert_true(g2.has_edge(2, 1))

    # then with duplicates removed,
    g2 = make_undirected(g, remove_dups=True)

    assert_equal(g2.num_nodes, g.num_nodes)
    assert_equal(g2.num_edges, 4)
    assert_true(g2.has_edge(0, 1))
    assert_true(g2.has_edge(1, 0))
    assert_true(g2.has_edge(1, 2))
    assert_true(g2.has_edge(2, 1))

    # and finally with weights.
    g2, edge_w2 = make_undirected(g, remove_dups=True, calc_weights=True)

    assert_equal(g2.num_nodes, g.num_nodes)
    assert_equal(g2.num_edges, 4)
    assert_true(g2.has_edge(0, 1))
    assert_true(g2.has_edge(1, 0))
    assert_true(g2.has_edge(1, 2))
    assert_true(g2.has_edge(2, 1))
    assert_equal(edge_w2[g2.first_edge_id(0,1)], 2)
    assert_equal(edge_w2[g2.first_edge_id(1,0)], 2)
    assert_equal(edge_w2[g2.first_edge_id(1,2)], 1)
    assert_equal(edge_w2[g2.first_edge_id(2,1)], 1)

def test_remove_duplicates():
    graph = Graph()
    graph.add_edge(0, 1)
    graph.add_edge(0, 1)
    graph.add_edge(1, 2)

    # First test without edge weights,
    g = graph.compressed()
    remove_duplicates(g)
    assert_equal(g.num_edges, 2)
    assert_true(g.has_edge(0, 1))
    assert_true(g.has_edge(1, 2))

    # then with edge weights.
    g = graph.compressed()
    edge_w = np.array([1,1,1], dtype=gint_dtype)
    remove_duplicates(g, edge_w)
    assert_equal(g.num_edges, 2)
    assert_true(g.has_edge(0, 1))
    assert_true(g.has_edge(1, 2))
    assert_equal(edge_w[g.first_edge_id(0,1)], 2)
    assert_equal(edge_w[g.first_edge_id(1,2)], 1)


def test_induced_subgraph():
    num_nodes = 6

    graph = Graph()
    for i in xrange(num_nodes-1):
        graph.add_edge(i, i + 1)
    graph.add_edge(1, 0)
    g = graph.compressed()

    # First test select array,
    select = np.array([True, True, True, False, False, True])
    g2 = induced_subgraph(g, select)
    assert_equal(g2.num_nodes, 4)
    assert_equal(g2.num_edges, 3)
    assert_true(g2.has_edge(0, 1))
    assert_true(g2.has_edge(1, 0))
    assert_true(g2.has_edge(1, 2))

    # then test select function.
    g2 = induced_subgraph(g, lambda i: select[i])
    assert_equal(g2.num_nodes, 4)
    assert_equal(g2.num_edges, 3)
    assert_true(g2.has_edge(0, 1))
    assert_true(g2.has_edge(1, 0))
    assert_true(g2.has_edge(1, 2))

    # Now the same with edge weights.
    edge_w = np.arange(g.num_edges, dtype=gint_dtype)
    g2, edge_w2 = induced_subgraph(g, select, edge_w)
    assert_equal(g2.num_nodes, 4)
    assert_equal(g2.num_edges, 3)
    assert_true(g2.has_edge(0, 1))
    assert_true(g2.has_edge(1, 0))
    assert_true(g2.has_edge(1, 2))
    assert_equal(edge_w[g.first_edge_id(0,1)],
                 edge_w2[g2.first_edge_id(0,1)])
    assert_equal(edge_w[g.first_edge_id(1,0)],
                 edge_w2[g2.first_edge_id(1,0)])
    assert_equal(edge_w[g.first_edge_id(1,2)],
                 edge_w2[g2.first_edge_id(1,2)])

    g2, edge_w2 = induced_subgraph(g, lambda i: select[i], edge_w)
    assert_equal(g2.num_nodes, 4)
    assert_equal(g2.num_edges, 3)
    assert_true(g2.has_edge(0, 1))
    assert_true(g2.has_edge(1, 0))
    assert_true(g2.has_edge(1, 2))
    assert_equal(edge_w[g.first_edge_id(0,1)],
                 edge_w2[g2.first_edge_id(0,1)])
    assert_equal(edge_w[g.first_edge_id(1,0)],
                 edge_w2[g2.first_edge_id(1,0)])
    assert_equal(edge_w[g.first_edge_id(1,2)],
                 edge_w2[g2.first_edge_id(1,2)])



# Copyright 2011-2013 Kwant authors.
#
# This file is part of Kwant.  It is subject to the license terms in the
# LICENSE file found in the top-level directory of this distribution and at
# http://kwant-project.org/license.  A list of Kwant authors can be found in
# the AUTHORS file at the top-level directory of this distribution and at
# http://kwant-project.org/authors.

"""Utilities to modify compressed graphs"""

__all__ = ['make_undirected', 'remove_duplicates', 'induced_subgraph',
           'print_graph']

import numpy as np
cimport numpy as np
from libc.stdlib cimport malloc, realloc, free
from libc.string cimport memset
cimport cython
from kwant.graph.defs cimport gint
from .defs import gint_dtype
from kwant.graph.core cimport CGraph, CGraph_malloc
from .core import CGraph, CGraph_malloc

@cython.boundscheck(False)
def make_undirected(CGraph gr, remove_dups=True, calc_weights=False):
    """undirected_graph(gr) expects a CGraph gr as input, which is interpreted
    as a directed graph, and returns a CGraph that is explicitely undirected,
    i.e. for every edge (i,j) there is also the edge (j,i). In the process, the
    function also removes all 'dangling' links, i.e. edges to or from
    negative node numbers.

    If remove_dups == True (default value is True), any duplicates of edges
    will be removed (this applies to the case where there are multiple edges
    (i,j), not to having (i,j) and (j,i)).

    The effect of the duplicate edges can be retained if calc_weights == True
    (default value is False), in which case a weight array is returned
    containing the multiplicity of the edges after the graph has been made
    undirected.

    As a (somewhat drastic but illustrative) example, if make_undirected is
    applied to a undirected graph, it will return the same graph again
    (possibly with the order of edges changed) and a weight array with 2
    everywhere.  (Of course, in this case one does not need to call
    make_undirected ...)

    make_undirected() will always return a one-way graph, regardless of
    whether the input was a two-way graph or not (NOTE: This
    restriction could be lifted, if necessary). In addition, the
    original edge_ids are lost -- the resulting graph will have
    edge_ids that are not related to the original ones. (NOTE: there
    certainly is a relation, but as long as no-one needs it it remains
    unspecified)
    """

    cdef gint i, j, p

    # The undirected graph will have twice as many edges than the directed one
    # (duplicates will be deleted afterwards).
    cdef CGraph_malloc ret = CGraph_malloc(False, False, gr.num_nodes,
                                           gr.heads_idxs[gr.num_nodes] * 2,
                                           0, 0)

    # In the following we build up the Graph directly in compressed format by
    # adding for every edge (i,j) [with i,j>=0] also the edge (j,i). Taking
    # care of possible doubling is done in a second step later.

    # Initialize the new index array:
    # First, compute a histogram of edges.
    memset(ret.heads_idxs, 0, (ret.num_nodes + 1) * sizeof(gint))

    # This is using Christoph's trick of building up the graph without
    # additional buffer array.
    cdef gint *buffer = ret.heads_idxs + 1

    for i in xrange(gr.num_nodes):
        for p in xrange(gr.heads_idxs[i], gr.heads_idxs[i+1]):
            if gr.heads[p] >= 0:
                buffer[i] += 1
                buffer[gr.heads[p]] += 1

    cdef gint s = 0
    for i in xrange(ret.num_nodes):
        s += buffer[i]
        buffer[i] = s - buffer[i]

    for i in xrange(gr.num_nodes):
       for p in xrange(gr.heads_idxs[i], gr.heads_idxs[i+1]):
           j = gr.heads[p]
           if j >= 0:
                ret.heads[buffer[i]] = j
                buffer[i] += 1
                ret.heads[buffer[j]] = i
                buffer[j] += 1

    ret.num_edges = ret.heads_idxs[ret.num_nodes]

    # Now remove duplicates if desired.
    cdef np.ndarray[gint, ndim=1] weights

    if calc_weights:
        weights = np.empty(ret.heads_idxs[ret.num_nodes], dtype=gint_dtype)
        weights[:] = 1

    if remove_dups:
        if calc_weights:
            remove_duplicates(ret, weights)
        else:
            remove_duplicates(ret)

    if calc_weights:
        return ret, weights
    else:
        return ret


@cython.boundscheck(False)
def remove_duplicates(CGraph gr, np.ndarray[gint, ndim=1] edge_weights=None):
    """Remove duplicate edges in the CGraph gr (this applies to the case where
    there are multiple edges (i,j), not to having (i,j) and (j,i)). This
    function modifes the graph in place.

    If edge_weights is provided, edge_weights is modified such that the new
    edge weights are the sum of the old edge weights if there are duplicate
    edges.

    This function only works on simple graphs (not two-way graphs), and
    it does not work on graphs which have a relation between the edge number
    (given by the order the edges are added) and the edge_id (given by the
    order the edges appear in the graph), see the documentation of CGraph.
    (Both restrictions could be lifted if necessary.) Furthermore, the
    function does not support negative node numbers, i.e. dangling links
    (the concept of being duplicate is more complicated there.)
    """
    cdef gint i, j , p, q, nnz
    cdef np.ndarray[gint, ndim=1] w

    if gr.twoway:
        raise RuntimeError("remove_duplicates does not support two-way "
                           "graphs")

    if gr.edge_ids_by_edge_nr:
        raise RuntimeError("remove_duplicates does not support graphs with "
                           "a relation between the edge number and the edge "
                           "id")

    # The array w will hold the position of head j in the heads array.
    w = np.empty(gr.num_nodes, dtype=gint_dtype)
    w[:]=-1

    nnz=0

    for i in xrange(gr.num_nodes):
        q = nnz

        for p in xrange(gr.heads_idxs[i], gr.heads_idxs[i+1]):
            j = gr.heads[p]

            # Check if we have found a previous entry (i,j).  (In this case w
            # will have an index larger than the indices of all previous edges
            # with tails < i, as stored in q.)
            if w[j] >= q:
                # entry is a duplicate
                if edge_weights != None:
                    edge_weights[w[j]] += edge_weights[p]
            else:
                w[j] = nnz
                gr.heads[nnz] = j
                nnz += 1

        # Fix the index array.
        gr.heads_idxs[i] = q

    # Finally the new number of nonzeros
    gr.heads_idxs[gr.num_nodes] = nnz
    gr.num_edges = nnz

    # Release memory that is not needed any more.
    gr.heads = <gint *>realloc(gr.heads, nnz * sizeof(gint))
    if not gr.heads:
        raise MemoryError

    if edge_weights != None:
        edge_weights.resize(nnz, refcheck=False)

@cython.boundscheck(False)
def induced_subgraph(CGraph gr, select,
                     np.ndarray[gint, ndim=1] edge_weights=None):
    """Return a subgraph of the CGraph gr by picking all nodes
    [0:gr.num_nodes] for which select is True. select can be either a
    NumPy array, or a function that takes the node number as
    input. This function returns a CGraph as well.

    The nodes in the new graph are again numbered sequentially from 0
    to num_nodes-1, where num_nodes is the number of nodes in the
    subgraph. The numbering is done such that the ordering of the node
    numbers in the original and the subgraph are preserved (i.e.
    if nodes n1 and n2 are both in the subgraph, and
    original node number of n1 < original node number of n2,
    then also subgraph node number n1 < subgraph node number n2).

    If edge_weights is provided, the function also returns the edge
    weights for the subgraph which are simply a subset of the original
    weights.

    This function returns a simple graph, regardless of whether the
    input was a two-way graph or not (NOTE: This restriction could be
    lifted, if necessary). Also, the resulting edge_ids are not
    related to the original ones in any way (NOTE: There certainly is
    a relation, but as long no-one needs it, we do not specify
    it). Also, negative nodes are discarded (NOTE: this restriction
    can also be lifted).
    """

    cdef np.ndarray[gint, ndim=1] indextab
    cdef CGraph_malloc subgr
    cdef np.ndarray[gint, ndim=1] sub_edge_weights
    cdef gint sub_num_nodes, sub_num_edges
    cdef gint i, iedge, edge_count

    # First figure out the new number of nodes.
    sub_num_nodes = 0
    indextab = np.empty(gr.num_nodes, dtype=gint_dtype)
    indextab[:] = -1

    # Pre-evaluating the select functions seems to be more than twice as fast
    # as calling select() repeatedly in the loop.  The thing is that one cannot
    # type ndarray as bool in Cython (yet) [Taking bint results in a strange
    # crash].  It would be possible to cast it into a cython type using
    # .astype(), but this didn't seem to make any relevant speed difference.
    if isinstance(select, np.ndarray):
        selecttab = select
    else:
        selecttab = select(np.arange(gr.num_nodes, dtype=gint_dtype))

    for i in xrange(gr.num_nodes):
        if selecttab[i]:
            indextab[i] = sub_num_nodes
            sub_num_nodes += 1

    # Now count the number of new edges.
    sub_num_edges = 0

    for i in xrange(gr.num_nodes):
        if indextab[i] > -1:
            for iedge in xrange(gr.heads_idxs[i], gr.heads_idxs[i + 1]):
                if indextab[gr.heads[iedge]] > -1:
                    sub_num_edges += 1

    # Allocate the new graph.
    subgr = CGraph_malloc(False, False, sub_num_nodes, sub_num_edges, 0, 0)

    if edge_weights != None:
        sub_edge_weights = np.empty(sub_num_edges, dtype=gint_dtype)

    # Now fill the new edge array.
    edge_count = 0

    for i in xrange(gr.num_nodes):
        if indextab[i]>-1:
            subgr.heads_idxs[indextab[i]] = edge_count
            for iedge in xrange(gr.heads_idxs[i], gr.heads_idxs[i+1]):
                if indextab[gr.heads[iedge]] > -1:
                    subgr.heads[edge_count] = indextab[gr.heads[iedge]]
                    if edge_weights != None:
                        sub_edge_weights[edge_count] = edge_weights[iedge]
                    edge_count += 1
    subgr.heads_idxs[sub_num_nodes] = edge_count

    subgr.num_edges = edge_count

    if edge_weights != None:
        return subgr, sub_edge_weights
    else:
        return subgr


def print_graph(gr):
    for i in xrange(gr.num_nodes):
        print i," -> ",
        for j in gr.out_neighbors(i):
            print j,
        print
